# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class EmailSpec(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'attachments': 'list[EmailAttachmentSpec]',
        'recipients': 'list[str]',
        'subject': 'str',
        'text': 'str'
    }

    attribute_map = {
        'attachments': 'attachments',
        'recipients': 'recipients',
        'subject': 'subject',
        'text': 'text'
    }

    def __init__(self, attachments=None, recipients=None, subject=None, text=None, local_vars_configuration=None):  # noqa: E501
        """EmailSpec - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._attachments = None
        self._recipients = None
        self._subject = None
        self._text = None
        self.discriminator = None

        if attachments is not None:
            self.attachments = attachments
        self.recipients = recipients
        self.subject = subject
        self.text = text

    @property
    def attachments(self):
        """
        S3 object(s) to send as attachment(s). Ensure the attachment size does not exceed the corresponding email service limits

        :return: The attachments of this EmailSpec. 
        :rtype: list[EmailAttachmentSpec]
        """
        return self._attachments

    @attachments.setter
    def attachments(self, attachments):
        """
        S3 object(s) to send as attachment(s). Ensure the attachment size does not exceed the corresponding email service limits

        :param attachments: The attachments of this EmailSpec.
        :type: list[EmailAttachmentSpec]
        """

        self._attachments = attachments

    @property
    def recipients(self):
        """
        Recipients of the email message. They must be members of this project. Entries are either in \"Name <email@email.com>\" or \"email@email.com\" format

        :return: The recipients of this EmailSpec. 
        :rtype: list[str]
        """
        return self._recipients

    @recipients.setter
    def recipients(self, recipients):
        """
        Recipients of the email message. They must be members of this project. Entries are either in \"Name <email@email.com>\" or \"email@email.com\" format

        :param recipients: The recipients of this EmailSpec.
        :type: list[str]
        """
        if self.local_vars_configuration.client_side_validation and recipients is None:  # noqa: E501
            raise ValueError("Invalid value for `recipients`, must not be `None`")  # noqa: E501

        self._recipients = recipients

    @property
    def subject(self):
        """
        Subject of the email message

        :return: The subject of this EmailSpec. 
        :rtype: str
        """
        return self._subject

    @subject.setter
    def subject(self, subject):
        """
        Subject of the email message

        :param subject: The subject of this EmailSpec.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and subject is None:  # noqa: E501
            raise ValueError("Invalid value for `subject`, must not be `None`")  # noqa: E501

        self._subject = subject

    @property
    def text(self):
        """
        Text body of the email message. HTML is currently not supported.

        :return: The text of this EmailSpec. 
        :rtype: str
        """
        return self._text

    @text.setter
    def text(self, text):
        """
        Text body of the email message. HTML is currently not supported.

        :param text: The text of this EmailSpec.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and text is None:  # noqa: E501
            raise ValueError("Invalid value for `text`, must not be `None`")  # noqa: E501

        self._text = text

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, EmailSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, EmailSpec):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class LocalizationIdQuery(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'elemental_ids': 'list[str]',
        'float_array': 'list[FloatArrayQuery]',
        'frame_state_ids': 'list[int]',
        'ids': 'list[int]',
        'media_ids': 'list[int]',
        'media_query': 'str',
        'object_search': 'AttributeOperationSpec',
        'state_ids': 'list[int]'
    }

    attribute_map = {
        'elemental_ids': 'elemental_ids',
        'float_array': 'float_array',
        'frame_state_ids': 'frame_state_ids',
        'ids': 'ids',
        'media_ids': 'media_ids',
        'media_query': 'media_query',
        'object_search': 'object_search',
        'state_ids': 'state_ids'
    }

    def __init__(self, elemental_ids=None, float_array=None, frame_state_ids=None, ids=None, media_ids=None, media_query=None, object_search=None, state_ids=None, local_vars_configuration=None):  # noqa: E501
        """LocalizationIdQuery - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._elemental_ids = None
        self._float_array = None
        self._frame_state_ids = None
        self._ids = None
        self._media_ids = None
        self._media_query = None
        self._object_search = None
        self._state_ids = None
        self.discriminator = None

        if elemental_ids is not None:
            self.elemental_ids = elemental_ids
        if float_array is not None:
            self.float_array = float_array
        if frame_state_ids is not None:
            self.frame_state_ids = frame_state_ids
        if ids is not None:
            self.ids = ids
        if media_ids is not None:
            self.media_ids = media_ids
        if media_query is not None:
            self.media_query = media_query
        if object_search is not None:
            self.object_search = object_search
        if state_ids is not None:
            self.state_ids = state_ids

    @property
    def elemental_ids(self):
        """
        Array of localizations to retrieve.

        :return: The elemental_ids of this LocalizationIdQuery. 
        :rtype: list[str]
        """
        return self._elemental_ids

    @elemental_ids.setter
    def elemental_ids(self, elemental_ids):
        """
        Array of localizations to retrieve.

        :param elemental_ids: The elemental_ids of this LocalizationIdQuery.
        :type: list[str]
        """

        self._elemental_ids = elemental_ids

    @property
    def float_array(self):
        """
        Searches on `float_array` attributes.

        :return: The float_array of this LocalizationIdQuery. 
        :rtype: list[FloatArrayQuery]
        """
        return self._float_array

    @float_array.setter
    def float_array(self, float_array):
        """
        Searches on `float_array` attributes.

        :param float_array: The float_array of this LocalizationIdQuery.
        :type: list[FloatArrayQuery]
        """

        self._float_array = float_array

    @property
    def frame_state_ids(self):
        """
        Array of frame state IDs for which localizations present in the same media/frame will be retrieved.

        :return: The frame_state_ids of this LocalizationIdQuery. 
        :rtype: list[int]
        """
        return self._frame_state_ids

    @frame_state_ids.setter
    def frame_state_ids(self, frame_state_ids):
        """
        Array of frame state IDs for which localizations present in the same media/frame will be retrieved.

        :param frame_state_ids: The frame_state_ids of this LocalizationIdQuery.
        :type: list[int]
        """

        self._frame_state_ids = frame_state_ids

    @property
    def ids(self):
        """
        Array of localization IDs to retrieve.

        :return: The ids of this LocalizationIdQuery. 
        :rtype: list[int]
        """
        return self._ids

    @ids.setter
    def ids(self, ids):
        """
        Array of localization IDs to retrieve.

        :param ids: The ids of this LocalizationIdQuery.
        :type: list[int]
        """

        self._ids = ids

    @property
    def media_ids(self):
        """
        Array of parent media IDs for which localizations should be retrieved.

        :return: The media_ids of this LocalizationIdQuery. 
        :rtype: list[int]
        """
        return self._media_ids

    @media_ids.setter
    def media_ids(self, media_ids):
        """
        Array of parent media IDs for which localizations should be retrieved.

        :param media_ids: The media_ids of this LocalizationIdQuery.
        :type: list[int]
        """

        self._media_ids = media_ids

    @property
    def media_query(self):
        """
        Query string used to filter media IDs. This can be used to avoid serialization and download of a media ID list.

        :return: The media_query of this LocalizationIdQuery. 
        :rtype: str
        """
        return self._media_query

    @media_query.setter
    def media_query(self, media_query):
        """
        Query string used to filter media IDs. This can be used to avoid serialization and download of a media ID list.

        :param media_query: The media_query of this LocalizationIdQuery.
        :type: str
        """

        self._media_query = media_query

    @property
    def object_search(self):
        """

        :return: The object_search of this LocalizationIdQuery. 
        :rtype: AttributeOperationSpec
        """
        return self._object_search

    @object_search.setter
    def object_search(self, object_search):
        """

        :param object_search: The object_search of this LocalizationIdQuery.
        :type: AttributeOperationSpec
        """

        self._object_search = object_search

    @property
    def state_ids(self):
        """
        Array of parent localization state (track) IDs for which localizations should be retrieved.

        :return: The state_ids of this LocalizationIdQuery. 
        :rtype: list[int]
        """
        return self._state_ids

    @state_ids.setter
    def state_ids(self, state_ids):
        """
        Array of parent localization state (track) IDs for which localizations should be retrieved.

        :param state_ids: The state_ids of this LocalizationIdQuery.
        :type: list[int]
        """

        self._state_ids = state_ids

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, LocalizationIdQuery):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, LocalizationIdQuery):
            return True

        return self.to_dict() != other.to_dict()



import hashlib
import os
import requests

def md5sum(fname, size=None):
    """ Computes md5sum-based fingerprint of a file contents. The return
        is the md5sum of the file contents (up to 100Mb) hashed along with
        the file size.

    :param fname: Path to file.
    :param size: Size of file. Will be computed if not given.
    :returns: md5 sum of the file.
    """
    CHUNK_SIZE = 100*1024*1024 # 100MB
    md5 = hashlib.md5()
    if fname.startswith("https://") or fname.startswith("http://"):
        HTTP_CHUNK = 10*1024*1024
        with requests.get(fname, stream=True) as r:
            r.raise_for_status()
            for count,chunk in enumerate(r.iter_content(chunk_size=HTTP_CHUNK)):
                md5.update(chunk)
                if (count + 1) * HTTP_CHUNK >= CHUNK_SIZE:
                    break # bail out after intended chunk size

        # Try to get the file size via HTTP, fall-back to provided size
        # if it doesn't work
        try:
            r = requests.head(fname)
            proposed_size = int(r.headers.get('content-length',0))
            if proposed_size:
                size = proposed_size
        except:
            if size is None or size <= 0:
                raise Exception("Must supply size if HTTP server doesn't support HEAD requests for size.")
    else:
        with open(fname, 'rb') as f:
            for chunk in iter(lambda: f.read(CHUNK_SIZE), b''):
                md5.update(chunk)
                break # Exit after one chunk

    # Compute file size if not given. (Already set for HTTP files)
    if (size is None or size <= 0) and os.path.exists(fname):
        size = os.stat(fname).st_size

    # If size still not computed, raise an exception.
    if size is None or size <= 0:
        raise Exception(f"Could not determine size of file {fname}!")

    # Salt in file size.
    out = hashlib.md5()
    out.update(md5.hexdigest().encode('utf-8') + str(size).encode('utf-8'))
    return out.hexdigest()



import os
import math

import requests

from ._download_file import _download_file
from ..openapi.tator_openapi.models import ImageDefinition, VideoDefinition

def _is_none_or_eq(variable, match):
    """  :returns: True if `variable` is None or equals `match` """
    return variable is None or variable == match

def _find_best_match(media_list, quality, codec_or_mime=None):
    """ Given a media list find the closest quality
        :param media_list: list of Media definitons either
                           :class:`tator.models.ImageDefinition` or
                           :class:`tator.models.VideoDefinition`.
        :param int quality: Vertical resolution to match (or None for best)
        :param str codec_or_mime: Specified codec or mime if multiple versions
                                  of 1 resolution exist. E.g. h264 or image/avif
    """
    # Without pulling in numpy we have to do this in an explicit loop
    available = [x.resolution[0] for x in media_list]

    match_idx = -1
    best_quality = max(available)
    quality = best_quality if quality is None else quality
    best_distance = best_quality+1
    for idx,val in enumerate(available):
        # Bounce out of any non matching codecs
        if codec_or_mime:
            media_info = media_list[idx]
            if type(media_info) == ImageDefinition and media_info.mime != codec_or_mime:
                continue
            if type(media_info) == VideoDefinition and media_info.codec != codec_or_mime:
                continue
        distance = abs(val-quality)
        if distance < best_distance:
            best_distance = distance
            match_idx = idx

    assert match_idx >= 0, "Unable to find match"

    return match_idx

def download_media(api, media, out_path, quality=None,media_type=None, codec_or_mime=None):
    """ Download a media file from Tator to an off-line location.

    Example:

    .. code-block:: python

        api = tator.get_api(host, token)
        media = api.get_media(media_id)
        out_path = f'/tmp/{media.name}'
        for progress in tator.util.download_media(api, media, out_path):
            print(f"Download progress: {progress}%")

        #download 720p version streaming
        for progress in tator.util.download_media(api,
                                                  media,
                                                  out_path,
                                                  720,
                                                  'streaming'):
            print(f"Download progress: {progress}%")


    :param api: :class:`tator.TatorApi` object.
    :param media: :class:`tator.Media` object.
    :param path-like out_path: Path to where to download.
    :param int quality: Attempts to fetch this resolution (defaults to best)
    :param str media_type: The desired item to download (archival, streaming,
                           image, thumbnail, or thumbnail_gif). Not all types
                           are valid for all media.
    :param str codec_or_mime: The desired codec or mime if multiple exist at
                              the same resolution. e.g. h264 for video or image/jpeg for images.
    :returns: Generator the yields progress (0-100).
    """
    assert media_type in [None,
                          'streaming',
                          'archival',
                          'image',
                          'thumbnail',
                          'thumbnail_gif']

    if media.media_files is not None:
        archival = media.media_files.archival
        archival = archival if archival else []
        streaming = media.media_files.streaming
        streaming = streaming if streaming else []
        image = media.media_files.image
        thumbnail = media.media_files.thumbnail
        thumbnail_gif = media.media_files.thumbnail_gif

        if _is_none_or_eq(media_type,"archival") and len(archival) > 0:
            url = archival[_find_best_match(archival,quality, codec_or_mime)].path
        elif _is_none_or_eq(media_type,"streaming") and len(streaming) > 0:
            url = streaming[_find_best_match(streaming,quality, codec_or_mime)].path
        elif _is_none_or_eq(media_type,"image") and image:
            url = image[_find_best_match(image,quality, codec_or_mime)].path
        elif media_type == "thumbnail":
            url = thumbnail[_find_best_match(thumbnail,quality)].path
        elif media_type == "thumbnail_gif":
            url = thumbnail_gif[_find_best_match(thumbnail_gif,quality)].path
        else:
            assert False, f"Unable to deduce download file media={media.id}"
    else:
        # Legacy way of using streaming prior to streaming
        # and images
        url = f"/media/{media.file}"
        if media.original:
            url = f"/data/raw/{media.original}"

    return _download_file(api, media.project, url, out_path)



import logging
import tator

import os
import requests
import tempfile

logger = logging.getLogger(__name__)

def test_media_manipulation(host: str,
                            token: str,
                            count_video: int):
  api = tator.get_api(host, token)
  media = api.get_media(count_video)
  
  original_streamer_count = len(media.media_files.streaming)
  original_archival_count = len(media.media_files.archival if media.media_files.archival else [])

  patch_in_res = media.media_files.streaming[0].resolution[0] - 16

  with tempfile.TemporaryDirectory() as td:
    VIDEO_URL = "https://github.com/cvisionai/rgb_test_videos/raw/v0.0.3/samples/count.mp4"
    r = requests.get(VIDEO_URL)
    video_path = os.path.join(td, "count.mp4")
    f = open(video_path, 'wb')
    for chunk in r.iter_content(chunk_size=1*1024*1024): 
        if chunk: # filter out keep-alive new chunks
            f.write(chunk)
    f.close()

    # Check ability to patch in a streaming file
    configuration = tator.models.ResolutionConfig(resolution=patch_in_res)
    tator.util.supplement_video_to_media(api, count_video, "streaming", configuration, video_path, force=False)
    media = api.get_media(count_video)
    new_streamer_count = len(media.media_files.streaming)
    assert new_streamer_count == original_streamer_count + 1

    found_it = False
    for streamer in media.media_files.streaming:
      if streamer.resolution[0] == patch_in_res:
        found_it = True
    assert(found_it)


    encode_configuration = tator.models.EncodeConfig(crf=40)
    archive_configuration = tator.models.ArchiveConfig(encode=encode_configuration)
    tator.util.supplement_video_to_media(api,count_video, "archival", archive_configuration, video_path, force=False)
    media = api.get_media(count_video)
    new_archival_count = len(media.media_files.archival)
    assert(new_archival_count == original_archival_count + 1)






"""Test SSRF protection for server-side requests."""

import os
import pytest
import tator
import uuid


# Helper to determine if SSRF protection should be tested
# Since we're setting SERVER_SIDE_REQUEST_ORIGINS in our test configurations,
# we expect these tests to pass unless explicitly disabled
def should_test_ssrf():
    """Check if SSRF protection tests should run."""
    # You could add logic here to skip tests in certain environments
    # For now, we always test since our test configs include the setting
    return True


def test_ssrf_protection_transcode(host, token, project, video_type):
    """Test that transcode endpoint blocks unauthorized origins."""
    api = tator.get_api(host, token)
    
    # Try to transcode from an unauthorized domain
    unauthorized_url = "https://www.w3schools.com/html/mov_bbb.mp4"
    
    # Attempt to create a transcode with an unauthorized URL
    with pytest.raises(tator.exceptions.ApiException) as exc_info:
        api.transcode(
            project=project,
            transcode_spec={
                "type": video_type,
                "gid": str(uuid.uuid4()),
                "uid": str(uuid.uuid4()),
                "url": unauthorized_url,
                "name": "test_ssrf_video.mp4",
                "section": "SSRF Test",
            }
        )
    
    # Verify we get a 403 Forbidden response
    assert exc_info.value.status == 403
    assert "not from an allowed origin" in str(exc_info.value.body).lower() or \
           "not from an allowed origin" in str(exc_info.value.reason).lower()


def test_ssrf_protection_import_image(host, token, project, image_type):
    """Test that import media endpoint blocks unauthorized origins."""
    api = tator.get_api(host, token)
    
    # Try to import from an unauthorized domain
    unauthorized_url = "https://www.w3schools.com/css/img_lights.jpg"
    
    # Attempt to import media with an unauthorized URL using the utility function
    with pytest.raises(tator.exceptions.ApiException) as exc_info:
        tator.util.import_media(api, image_type, unauthorized_url)
    
    # Verify we get a 403 Forbidden response
    assert exc_info.value.status == 403
    assert "not from an allowed origin" in str(exc_info.value.body).lower() or \
           "not from an allowed origin" in str(exc_info.value.reason).lower()


def test_ssrf_protection_import_video(host, token, project, video_type):
    """Test that import media endpoint blocks unauthorized origins."""
    api = tator.get_api(host, token)
    
    # Try to import from an unauthorized domain
    unauthorized_url = "https://www.w3schools.com/html/mov_bbb.mp4"
    
    # Attempt to import media with an unauthorized URL using the utility function
    with pytest.raises(tator.exceptions.ApiException) as exc_info:
        tator.util.import_media(api, video_type, unauthorized_url)
    
    # Verify we get a 403 Forbidden response
    assert exc_info.value.status == 403
    assert "not from an allowed origin" in str(exc_info.value.body).lower() or \
           "not from an allowed origin" in str(exc_info.value.reason).lower()


def test_ssrf_protection_temporary_file(host, token, project):
    """Test that temporary file endpoint blocks unauthorized origins."""
    api = tator.get_api(host, token)
    
    # Try to create a temporary file from an unauthorized domain
    unauthorized_url = "https://www.w3schools.com/xml/note.xml"
    
    # Attempt to create a temporary file with an unauthorized URL
    with pytest.raises(tator.exceptions.ApiException) as exc_info:
        api.create_temporary_file(
            project=project,
            temporary_file_spec={
                "url": unauthorized_url,
                "name": "test_ssrf_file.txt",
                "lookup": "ssrf_test_lookup",
                "hours": 1,
            }
        )
    
    # Verify we get a 403 Forbidden response
    assert exc_info.value.status == 403
    assert "not from an allowed origin" in str(exc_info.value.body).lower() or \
           "not from an allowed origin" in str(exc_info.value.reason).lower()


def test_ssrf_protection_save_generic_file(host, token, project):
    """Test that save generic file endpoint blocks unauthorized origins."""
    api = tator.get_api(host, token)
    
    # Try to create a temporary file from an unauthorized domain
    unauthorized_url = "https://www.w3schools.com/xml/note.xml"
    
    # Attempt to create a temporary file with an unauthorized URL
    with pytest.raises(tator.exceptions.ApiException) as exc_info:
        api.save_generic_file(
            project=project,
            generic_file_spec={
                "upload_url": unauthorized_url,
                "url": unauthorized_url,
                "project": project,
            }
        )
    
    # Verify we get a 403 Forbidden response
    assert exc_info.value.status == 403
    assert "not from an allowed origin" in str(exc_info.value.body).lower() or \
           "not from an allowed origin" in str(exc_info.value.reason).lower()


def test_ssrf_protection_hosted_template(host, token, organization):
    """Test that hosted template endpoint blocks unauthorized origins."""
    api = tator.get_api(host, token)
    
    # Try to create a hosted template from an unauthorized domain (using a real YAML file that exists)
    unauthorized_url = "https://www.w3schools.com/xml/simple.xml"
    
    # Attempt to create a hosted template with an unauthorized URL
    with pytest.raises(tator.exceptions.ApiException) as exc_info:
        api.create_hosted_template(
            organization=organization,
            hosted_template_spec={
                "name": "SSRF Test Template",
                "url": unauthorized_url,
            }
        )
    
    # Verify we get a 403 Forbidden response
    assert exc_info.value.status == 403
    assert "not from an allowed origin" in str(exc_info.value.body).lower() or \
           "not from an allowed origin" in str(exc_info.value.reason).lower()





# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class BucketUpdate(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'archive_sc': 'str',
        'config': 'object',
        'effective_permission': 'int',
        'external_host': 'str',
        'live_sc': 'str',
        'name': 'str',
        'store_type': 'str'
    }

    attribute_map = {
        'archive_sc': 'archive_sc',
        'config': 'config',
        'effective_permission': 'effective_permission',
        'external_host': 'external_host',
        'live_sc': 'live_sc',
        'name': 'name',
        'store_type': 'store_type'
    }

    def __init__(self, archive_sc=None, config=None, effective_permission=None, external_host=None, live_sc=None, name=None, store_type=None, local_vars_configuration=None):  # noqa: E501
        """BucketUpdate - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._archive_sc = None
        self._config = None
        self._effective_permission = None
        self._external_host = None
        self._live_sc = None
        self._name = None
        self._store_type = None
        self.discriminator = None

        if archive_sc is not None:
            self.archive_sc = archive_sc
        if config is not None:
            self.config = config
        if effective_permission is not None:
            self.effective_permission = effective_permission
        if external_host is not None:
            self.external_host = external_host
        if live_sc is not None:
            self.live_sc = live_sc
        if name is not None:
            self.name = name
        if store_type is not None:
            self.store_type = store_type

    @property
    def archive_sc(self):
        """
        Storage class in which archived objects live.

        :return: The archive_sc of this BucketUpdate. 
        :rtype: str
        """
        return self._archive_sc

    @archive_sc.setter
    def archive_sc(self, archive_sc):
        """
        Storage class in which archived objects live.

        :param archive_sc: The archive_sc of this BucketUpdate.
        :type: str
        """
        allowed_values = ["STANDARD", "DEEP_ARCHIVE", "COLDLINE"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and archive_sc not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `archive_sc` ({0}), must be one of {1}"  # noqa: E501
                .format(archive_sc, allowed_values)
            )

        self._archive_sc = archive_sc

    @property
    def config(self):
        """
        JSON string containing cloud storage credentials.

        :return: The config of this BucketUpdate. 
        :rtype: object
        """
        return self._config

    @config.setter
    def config(self, config):
        """
        JSON string containing cloud storage credentials.

        :param config: The config of this BucketUpdate.
        :type: object
        """

        self._config = config

    @property
    def effective_permission(self):
        """
        Effective permission mask for this entity.

        :return: The effective_permission of this BucketUpdate. 
        :rtype: int
        """
        return self._effective_permission

    @effective_permission.setter
    def effective_permission(self, effective_permission):
        """
        Effective permission mask for this entity.

        :param effective_permission: The effective_permission of this BucketUpdate.
        :type: int
        """

        self._effective_permission = effective_permission

    @property
    def external_host(self):
        """
        The proxy host for presigned urls.

        :return: The external_host of this BucketUpdate. 
        :rtype: str
        """
        return self._external_host

    @external_host.setter
    def external_host(self, external_host):
        """
        The proxy host for presigned urls.

        :param external_host: The external_host of this BucketUpdate.
        :type: str
        """

        self._external_host = external_host

    @property
    def live_sc(self):
        """
        Storage class in which live objects live.

        :return: The live_sc of this BucketUpdate. 
        :rtype: str
        """
        return self._live_sc

    @live_sc.setter
    def live_sc(self, live_sc):
        """
        Storage class in which live objects live.

        :param live_sc: The live_sc of this BucketUpdate.
        :type: str
        """
        allowed_values = ["STANDARD"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and live_sc not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `live_sc` ({0}), must be one of {1}"  # noqa: E501
                .format(live_sc, allowed_values)
            )

        self._live_sc = live_sc

    @property
    def name(self):
        """
        Bucket name.

        :return: The name of this BucketUpdate. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Bucket name.

        :param name: The name of this BucketUpdate.
        :type: str
        """

        self._name = name

    @property
    def store_type(self):
        """
        Type of object store on which the bucket is hosted.

        :return: The store_type of this BucketUpdate. 
        :rtype: str
        """
        return self._store_type

    @store_type.setter
    def store_type(self, store_type):
        """
        Type of object store on which the bucket is hosted.

        :param store_type: The store_type of this BucketUpdate.
        :type: str
        """
        allowed_values = ["AWS", "MINIO", "GCP", "OCI", "S3BASIC", "AZURE"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and store_type not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `store_type` ({0}), must be one of {1}"  # noqa: E501
                .format(store_type, allowed_values)
            )

        self._store_type = store_type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BucketUpdate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, BucketUpdate):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class Media(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'archive_state': 'str',
        'archive_status_date': 'datetime',
        'attributes': 'dict(str, object)',
        'codec': 'str',
        'created_by': 'int',
        'created_datetime': 'str',
        'effective_permission': 'int',
        'elemental_id': 'str',
        'fps': 'float',
        'gid': 'str',
        'height': 'int',
        'id': 'int',
        'incident': 'int',
        'last_edit_end': 'datetime',
        'last_edit_start': 'datetime',
        'master_section': 'int',
        'md5': 'str',
        'media_files': 'MediaFiles',
        'modified_by': 'int',
        'modified_datetime': 'str',
        'name': 'str',
        'num_frames': 'int',
        'primary_section': 'int',
        'project': 'int',
        'summary_level': 'int',
        'type': 'int',
        'uid': 'str',
        'width': 'int'
    }

    attribute_map = {
        'archive_state': 'archive_state',
        'archive_status_date': 'archive_status_date',
        'attributes': 'attributes',
        'codec': 'codec',
        'created_by': 'created_by',
        'created_datetime': 'created_datetime',
        'effective_permission': 'effective_permission',
        'elemental_id': 'elemental_id',
        'fps': 'fps',
        'gid': 'gid',
        'height': 'height',
        'id': 'id',
        'incident': 'incident',
        'last_edit_end': 'last_edit_end',
        'last_edit_start': 'last_edit_start',
        'master_section': 'master_section',
        'md5': 'md5',
        'media_files': 'media_files',
        'modified_by': 'modified_by',
        'modified_datetime': 'modified_datetime',
        'name': 'name',
        'num_frames': 'num_frames',
        'primary_section': 'primary_section',
        'project': 'project',
        'summary_level': 'summary_level',
        'type': 'type',
        'uid': 'uid',
        'width': 'width'
    }

    def __init__(self, archive_state=None, archive_status_date=None, attributes=None, codec=None, created_by=None, created_datetime=None, effective_permission=None, elemental_id=None, fps=None, gid=None, height=None, id=None, incident=None, last_edit_end=None, last_edit_start=None, master_section=None, md5=None, media_files=None, modified_by=None, modified_datetime=None, name=None, num_frames=None, primary_section=None, project=None, summary_level=None, type=None, uid=None, width=None, local_vars_configuration=None):  # noqa: E501
        """Media - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._archive_state = None
        self._archive_status_date = None
        self._attributes = None
        self._codec = None
        self._created_by = None
        self._created_datetime = None
        self._effective_permission = None
        self._elemental_id = None
        self._fps = None
        self._gid = None
        self._height = None
        self._id = None
        self._incident = None
        self._last_edit_end = None
        self._last_edit_start = None
        self._master_section = None
        self._md5 = None
        self._media_files = None
        self._modified_by = None
        self._modified_datetime = None
        self._name = None
        self._num_frames = None
        self._primary_section = None
        self._project = None
        self._summary_level = None
        self._type = None
        self._uid = None
        self._width = None
        self.discriminator = None

        if archive_state is not None:
            self.archive_state = archive_state
        if archive_status_date is not None:
            self.archive_status_date = archive_status_date
        if attributes is not None:
            self.attributes = attributes
        if codec is not None:
            self.codec = codec
        if created_by is not None:
            self.created_by = created_by
        if created_datetime is not None:
            self.created_datetime = created_datetime
        self.effective_permission = effective_permission
        self.elemental_id = elemental_id
        if fps is not None:
            self.fps = fps
        if gid is not None:
            self.gid = gid
        if height is not None:
            self.height = height
        if id is not None:
            self.id = id
        self.incident = incident
        if last_edit_end is not None:
            self.last_edit_end = last_edit_end
        if last_edit_start is not None:
            self.last_edit_start = last_edit_start
        if master_section is not None:
            self.master_section = master_section
        if md5 is not None:
            self.md5 = md5
        if media_files is not None:
            self.media_files = media_files
        if modified_by is not None:
            self.modified_by = modified_by
        if modified_datetime is not None:
            self.modified_datetime = modified_datetime
        if name is not None:
            self.name = name
        if num_frames is not None:
            self.num_frames = num_frames
        if primary_section is not None:
            self.primary_section = primary_section
        if project is not None:
            self.project = project
        if summary_level is not None:
            self.summary_level = summary_level
        if type is not None:
            self.type = type
        if uid is not None:
            self.uid = uid
        if width is not None:
            self.width = width

    @property
    def archive_state(self):
        """
        The current archival state of the media.

        :return: The archive_state of this Media. 
        :rtype: str
        """
        return self._archive_state

    @archive_state.setter
    def archive_state(self, archive_state):
        """
        The current archival state of the media.

        :param archive_state: The archive_state of this Media.
        :type: str
        """
        allowed_values = ["archived", "to_archive", "live", "to_live"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and archive_state not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `archive_state` ({0}), must be one of {1}"  # noqa: E501
                .format(archive_state, allowed_values)
            )

        self._archive_state = archive_state

    @property
    def archive_status_date(self):
        """
        Datetime of the last change to the `archive_state` field.

        :return: The archive_status_date of this Media. 
        :rtype: datetime
        """
        return self._archive_status_date

    @archive_status_date.setter
    def archive_status_date(self, archive_status_date):
        """
        Datetime of the last change to the `archive_state` field.

        :param archive_status_date: The archive_status_date of this Media.
        :type: datetime
        """

        self._archive_status_date = archive_status_date

    @property
    def attributes(self):
        """
        Object containing attribute values.

        :return: The attributes of this Media. 
        :rtype: dict(str, object)
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """
        Object containing attribute values.

        :param attributes: The attributes of this Media.
        :type: dict(str, object)
        """

        self._attributes = attributes

    @property
    def codec(self):
        """
        Codec for videos.

        :return: The codec of this Media. 
        :rtype: str
        """
        return self._codec

    @codec.setter
    def codec(self, codec):
        """
        Codec for videos.

        :param codec: The codec of this Media.
        :type: str
        """

        self._codec = codec

    @property
    def created_by(self):
        """
        Unique integer identifying user who created this media.

        :return: The created_by of this Media. 
        :rtype: int
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """
        Unique integer identifying user who created this media.

        :param created_by: The created_by of this Media.
        :type: int
        """

        self._created_by = created_by

    @property
    def created_datetime(self):
        """
        Datetime when this media was created.

        :return: The created_datetime of this Media. 
        :rtype: str
        """
        return self._created_datetime

    @created_datetime.setter
    def created_datetime(self, created_datetime):
        """
        Datetime when this media was created.

        :param created_datetime: The created_datetime of this Media.
        :type: str
        """

        self._created_datetime = created_datetime

    @property
    def effective_permission(self):
        """
        Calculated permission for this value

        :return: The effective_permission of this Media. 
        :rtype: int
        """
        return self._effective_permission

    @effective_permission.setter
    def effective_permission(self, effective_permission):
        """
        Calculated permission for this value

        :param effective_permission: The effective_permission of this Media.
        :type: int
        """

        self._effective_permission = effective_permission

    @property
    def elemental_id(self):
        """
        The elemental ID of the object.

        :return: The elemental_id of this Media. 
        :rtype: str
        """
        return self._elemental_id

    @elemental_id.setter
    def elemental_id(self, elemental_id):
        """
        The elemental ID of the object.

        :param elemental_id: The elemental_id of this Media.
        :type: str
        """

        self._elemental_id = elemental_id

    @property
    def fps(self):
        """
        Frame rate for videos.

        :return: The fps of this Media. 
        :rtype: float
        """
        return self._fps

    @fps.setter
    def fps(self, fps):
        """
        Frame rate for videos.

        :param fps: The fps of this Media.
        :type: float
        """

        self._fps = fps

    @property
    def gid(self):
        """
        Group ID for the upload group of this media.

        :return: The gid of this Media. 
        :rtype: str
        """
        return self._gid

    @gid.setter
    def gid(self, gid):
        """
        Group ID for the upload group of this media.

        :param gid: The gid of this Media.
        :type: str
        """

        self._gid = gid

    @property
    def height(self):
        """
        Vertical resolution in pixels.

        :return: The height of this Media. 
        :rtype: int
        """
        return self._height

    @height.setter
    def height(self, height):
        """
        Vertical resolution in pixels.

        :param height: The height of this Media.
        :type: int
        """

        self._height = height

    @property
    def id(self):
        """
        Unique integer identifying this media.

        :return: The id of this Media. 
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique integer identifying this media.

        :param id: The id of this Media.
        :type: int
        """

        self._id = id

    @property
    def incident(self):
        """
        If doing a related search, will contain the number of matching metadata hits in this media.

        :return: The incident of this Media. 
        :rtype: int
        """
        return self._incident

    @incident.setter
    def incident(self, incident):
        """
        If doing a related search, will contain the number of matching metadata hits in this media.

        :param incident: The incident of this Media.
        :type: int
        """

        self._incident = incident

    @property
    def last_edit_end(self):
        """
        Datetime of the end of the session when this media or its annotations were last edited.

        :return: The last_edit_end of this Media. 
        :rtype: datetime
        """
        return self._last_edit_end

    @last_edit_end.setter
    def last_edit_end(self, last_edit_end):
        """
        Datetime of the end of the session when this media or its annotations were last edited.

        :param last_edit_end: The last_edit_end of this Media.
        :type: datetime
        """

        self._last_edit_end = last_edit_end

    @property
    def last_edit_start(self):
        """
        Datetime of the start of the session when this media or its annotations were last edited.

        :return: The last_edit_start of this Media. 
        :rtype: datetime
        """
        return self._last_edit_start

    @last_edit_start.setter
    def last_edit_start(self, last_edit_start):
        """
        Datetime of the start of the session when this media or its annotations were last edited.

        :param last_edit_start: The last_edit_start of this Media.
        :type: datetime
        """

        self._last_edit_start = last_edit_start

    @property
    def master_section(self):
        """
        Unique ID to define the ACL origin of this object. If not set defaults to the master section of the primary_section.

        :return: The master_section of this Media. 
        :rtype: int
        """
        return self._master_section

    @master_section.setter
    def master_section(self, master_section):
        """
        Unique ID to define the ACL origin of this object. If not set defaults to the master section of the primary_section.

        :param master_section: The master_section of this Media.
        :type: int
        """

        self._master_section = master_section

    @property
    def md5(self):
        """
        MD5 checksum of the media file.

        :return: The md5 of this Media. 
        :rtype: str
        """
        return self._md5

    @md5.setter
    def md5(self, md5):
        """
        MD5 checksum of the media file.

        :param md5: The md5 of this Media.
        :type: str
        """

        self._md5 = md5

    @property
    def media_files(self):
        """

        :return: The media_files of this Media. 
        :rtype: MediaFiles
        """
        return self._media_files

    @media_files.setter
    def media_files(self, media_files):
        """

        :param media_files: The media_files of this Media.
        :type: MediaFiles
        """

        self._media_files = media_files

    @property
    def modified_by(self):
        """
        Unique integer identifying user who last modified this media.

        :return: The modified_by of this Media. 
        :rtype: int
        """
        return self._modified_by

    @modified_by.setter
    def modified_by(self, modified_by):
        """
        Unique integer identifying user who last modified this media.

        :param modified_by: The modified_by of this Media.
        :type: int
        """

        self._modified_by = modified_by

    @property
    def modified_datetime(self):
        """
        Datetime when this media was last modified.

        :return: The modified_datetime of this Media. 
        :rtype: str
        """
        return self._modified_datetime

    @modified_datetime.setter
    def modified_datetime(self, modified_datetime):
        """
        Datetime when this media was last modified.

        :param modified_datetime: The modified_datetime of this Media.
        :type: str
        """

        self._modified_datetime = modified_datetime

    @property
    def name(self):
        """
        Name of the media.

        :return: The name of this Media. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the media.

        :param name: The name of this Media.
        :type: str
        """

        self._name = name

    @property
    def num_frames(self):
        """
        Number of frames for videos.

        :return: The num_frames of this Media. 
        :rtype: int
        """
        return self._num_frames

    @num_frames.setter
    def num_frames(self, num_frames):
        """
        Number of frames for videos.

        :param num_frames: The num_frames of this Media.
        :type: int
        """

        self._num_frames = num_frames

    @property
    def primary_section(self):
        """
        Unique ID for folder section containing this media, if any. When patching, use -1 to indicate no section.

        :return: The primary_section of this Media. 
        :rtype: int
        """
        return self._primary_section

    @primary_section.setter
    def primary_section(self, primary_section):
        """
        Unique ID for folder section containing this media, if any. When patching, use -1 to indicate no section.

        :param primary_section: The primary_section of this Media.
        :type: int
        """

        self._primary_section = primary_section

    @property
    def project(self):
        """
        Unique integer identifying project of this media.

        :return: The project of this Media. 
        :rtype: int
        """
        return self._project

    @project.setter
    def project(self, project):
        """
        Unique integer identifying project of this media.

        :param project: The project of this Media.
        :type: int
        """

        self._project = project

    @property
    def summary_level(self):
        """
        If supplied, this video is best summarized at this frame interval

        :return: The summary_level of this Media. 
        :rtype: int
        """
        return self._summary_level

    @summary_level.setter
    def summary_level(self, summary_level):
        """
        If supplied, this video is best summarized at this frame interval

        :param summary_level: The summary_level of this Media.
        :type: int
        """

        self._summary_level = summary_level

    @property
    def type(self):
        """
        Unique integer identifying entity type of this media.

        :return: The type of this Media. 
        :rtype: int
        """
        return self._type

    @type.setter
    def type(self, type):
        """
        Unique integer identifying entity type of this media.

        :param type: The type of this Media.
        :type: int
        """

        self._type = type

    @property
    def uid(self):
        """
        Unique ID for the upload of this media.

        :return: The uid of this Media. 
        :rtype: str
        """
        return self._uid

    @uid.setter
    def uid(self, uid):
        """
        Unique ID for the upload of this media.

        :param uid: The uid of this Media.
        :type: str
        """

        self._uid = uid

    @property
    def width(self):
        """
        Horizontal resolution in pixels.

        :return: The width of this Media. 
        :rtype: int
        """
        return self._width

    @width.setter
    def width(self, width):
        """
        Horizontal resolution in pixels.

        :param width: The width of this Media.
        :type: int
        """

        self._width = width

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Media):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Media):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

# flake8: noqa

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

__version__ = "1.6.5"

# import apis into sdk package
from tator.openapi.tator_openapi.api.tator_api import TatorApi

# import ApiClient
from tator.openapi.tator_openapi.api_client import ApiClient
from tator.openapi.tator_openapi.configuration import Configuration
from tator.openapi.tator_openapi.exceptions import OpenApiException
from tator.openapi.tator_openapi.exceptions import ApiTypeError
from tator.openapi.tator_openapi.exceptions import ApiValueError
from tator.openapi.tator_openapi.exceptions import ApiKeyError
from tator.openapi.tator_openapi.exceptions import ApiException
# import models into sdk package
from tator.openapi.tator_openapi.models.affiliation import Affiliation
from tator.openapi.tator_openapi.models.affiliation_spec import AffiliationSpec
from tator.openapi.tator_openapi.models.affiliation_update import AffiliationUpdate
from tator.openapi.tator_openapi.models.algorithm import Algorithm
from tator.openapi.tator_openapi.models.algorithm_manifest import AlgorithmManifest
from tator.openapi.tator_openapi.models.algorithm_manifest_spec import AlgorithmManifestSpec
from tator.openapi.tator_openapi.models.algorithm_parameter import AlgorithmParameter
from tator.openapi.tator_openapi.models.algorithm_spec import AlgorithmSpec
from tator.openapi.tator_openapi.models.announcement import Announcement
from tator.openapi.tator_openapi.models.applet import Applet
from tator.openapi.tator_openapi.models.applet_spec import AppletSpec
from tator.openapi.tator_openapi.models.archive_config import ArchiveConfig
from tator.openapi.tator_openapi.models.attribute_combinator_spec import AttributeCombinatorSpec
from tator.openapi.tator_openapi.models.attribute_filter_spec import AttributeFilterSpec
from tator.openapi.tator_openapi.models.attribute_operation_spec import AttributeOperationSpec
from tator.openapi.tator_openapi.models.attribute_type import AttributeType
from tator.openapi.tator_openapi.models.attribute_type_delete import AttributeTypeDelete
from tator.openapi.tator_openapi.models.attribute_type_spec import AttributeTypeSpec
from tator.openapi.tator_openapi.models.attribute_type_update import AttributeTypeUpdate
from tator.openapi.tator_openapi.models.attribute_type_update_attribute_type_update import AttributeTypeUpdateAttributeTypeUpdate
from tator.openapi.tator_openapi.models.audio_definition import AudioDefinition
from tator.openapi.tator_openapi.models.autocomplete_service import AutocompleteService
from tator.openapi.tator_openapi.models.auxiliary_file_definition import AuxiliaryFileDefinition
from tator.openapi.tator_openapi.models.bad_request_response import BadRequestResponse
from tator.openapi.tator_openapi.models.bookmark import Bookmark
from tator.openapi.tator_openapi.models.bookmark_spec import BookmarkSpec
from tator.openapi.tator_openapi.models.bookmark_update import BookmarkUpdate
from tator.openapi.tator_openapi.models.bucket import Bucket
from tator.openapi.tator_openapi.models.bucket_azure_config import BucketAzureConfig
from tator.openapi.tator_openapi.models.bucket_gcp_config import BucketGCPConfig
from tator.openapi.tator_openapi.models.bucket_oci_config import BucketOCIConfig
from tator.openapi.tator_openapi.models.bucket_oci_native_config import BucketOCINativeConfig
from tator.openapi.tator_openapi.models.bucket_s3_config import BucketS3Config
from tator.openapi.tator_openapi.models.bucket_spec import BucketSpec
from tator.openapi.tator_openapi.models.bucket_update import BucketUpdate
from tator.openapi.tator_openapi.models.change_log import ChangeLog
from tator.openapi.tator_openapi.models.change_log_description_of_change import ChangeLogDescriptionOfChange
from tator.openapi.tator_openapi.models.change_log_description_of_change_new import ChangeLogDescriptionOfChangeNew
from tator.openapi.tator_openapi.models.clone_media_spec import CloneMediaSpec
from tator.openapi.tator_openapi.models.color_map import ColorMap
from tator.openapi.tator_openapi.models.concat_definition import ConcatDefinition
from tator.openapi.tator_openapi.models.create_list_response import CreateListResponse
from tator.openapi.tator_openapi.models.create_response import CreateResponse
from tator.openapi.tator_openapi.models.credentials import Credentials
from tator.openapi.tator_openapi.models.download_info import DownloadInfo
from tator.openapi.tator_openapi.models.download_info_spec import DownloadInfoSpec
from tator.openapi.tator_openapi.models.email_attachment_spec import EmailAttachmentSpec
from tator.openapi.tator_openapi.models.email_spec import EmailSpec
from tator.openapi.tator_openapi.models.encode_config import EncodeConfig
from tator.openapi.tator_openapi.models.favorite import Favorite
from tator.openapi.tator_openapi.models.favorite_spec import FavoriteSpec
from tator.openapi.tator_openapi.models.favorite_update import FavoriteUpdate
from tator.openapi.tator_openapi.models.feed_definition import FeedDefinition
from tator.openapi.tator_openapi.models.file import File
from tator.openapi.tator_openapi.models.file_spec import FileSpec
from tator.openapi.tator_openapi.models.file_type import FileType
from tator.openapi.tator_openapi.models.file_type_spec import FileTypeSpec
from tator.openapi.tator_openapi.models.file_type_update import FileTypeUpdate
from tator.openapi.tator_openapi.models.file_update import FileUpdate
from tator.openapi.tator_openapi.models.fill import Fill
from tator.openapi.tator_openapi.models.float_array_query import FloatArrayQuery
from tator.openapi.tator_openapi.models.generic_file import GenericFile
from tator.openapi.tator_openapi.models.generic_file_spec import GenericFileSpec
from tator.openapi.tator_openapi.models.get_cloned_media_response import GetClonedMediaResponse
from tator.openapi.tator_openapi.models.group import Group
from tator.openapi.tator_openapi.models.group_spec import GroupSpec
from tator.openapi.tator_openapi.models.group_update_spec import GroupUpdateSpec
from tator.openapi.tator_openapi.models.hosted_template import HostedTemplate
from tator.openapi.tator_openapi.models.hosted_template_spec import HostedTemplateSpec
from tator.openapi.tator_openapi.models.image_definition import ImageDefinition
from tator.openapi.tator_openapi.models.inline_response200 import InlineResponse200
from tator.openapi.tator_openapi.models.inline_response2001 import InlineResponse2001
from tator.openapi.tator_openapi.models.inline_response2002 import InlineResponse2002
from tator.openapi.tator_openapi.models.inline_response201 import InlineResponse201
from tator.openapi.tator_openapi.models.invitation import Invitation
from tator.openapi.tator_openapi.models.invitation_spec import InvitationSpec
from tator.openapi.tator_openapi.models.invitation_update import InvitationUpdate
from tator.openapi.tator_openapi.models.job import Job
from tator.openapi.tator_openapi.models.job_cluster import JobCluster
from tator.openapi.tator_openapi.models.job_cluster_spec import JobClusterSpec
from tator.openapi.tator_openapi.models.job_node import JobNode
from tator.openapi.tator_openapi.models.job_spec import JobSpec
from tator.openapi.tator_openapi.models.job_spec_failure_email_spec import JobSpecFailureEmailSpec
from tator.openapi.tator_openapi.models.leaf import Leaf
from tator.openapi.tator_openapi.models.leaf_bulk_update import LeafBulkUpdate
from tator.openapi.tator_openapi.models.leaf_id_query import LeafIdQuery
from tator.openapi.tator_openapi.models.leaf_spec import LeafSpec
from tator.openapi.tator_openapi.models.leaf_suggestion import LeafSuggestion
from tator.openapi.tator_openapi.models.leaf_type import LeafType
from tator.openapi.tator_openapi.models.leaf_type_spec import LeafTypeSpec
from tator.openapi.tator_openapi.models.leaf_type_update import LeafTypeUpdate
from tator.openapi.tator_openapi.models.leaf_update import LeafUpdate
from tator.openapi.tator_openapi.models.live_definition import LiveDefinition
from tator.openapi.tator_openapi.models.live_update_definition import LiveUpdateDefinition
from tator.openapi.tator_openapi.models.localization import Localization
from tator.openapi.tator_openapi.models.localization_bulk_delete import LocalizationBulkDelete
from tator.openapi.tator_openapi.models.localization_bulk_update import LocalizationBulkUpdate
from tator.openapi.tator_openapi.models.localization_delete import LocalizationDelete
from tator.openapi.tator_openapi.models.localization_id_query import LocalizationIdQuery
from tator.openapi.tator_openapi.models.localization_spec import LocalizationSpec
from tator.openapi.tator_openapi.models.localization_type import LocalizationType
from tator.openapi.tator_openapi.models.localization_type_spec import LocalizationTypeSpec
from tator.openapi.tator_openapi.models.localization_type_update import LocalizationTypeUpdate
from tator.openapi.tator_openapi.models.localization_update import LocalizationUpdate
from tator.openapi.tator_openapi.models.media import Media
from tator.openapi.tator_openapi.models.media_bulk_update import MediaBulkUpdate
from tator.openapi.tator_openapi.models.media_files import MediaFiles
from tator.openapi.tator_openapi.models.media_id_query import MediaIdQuery
from tator.openapi.tator_openapi.models.media_next import MediaNext
from tator.openapi.tator_openapi.models.media_prev import MediaPrev
from tator.openapi.tator_openapi.models.media_spec import MediaSpec
from tator.openapi.tator_openapi.models.media_stats import MediaStats
from tator.openapi.tator_openapi.models.media_type import MediaType
from tator.openapi.tator_openapi.models.media_type_spec import MediaTypeSpec
from tator.openapi.tator_openapi.models.media_type_update import MediaTypeUpdate
from tator.openapi.tator_openapi.models.media_update import MediaUpdate
from tator.openapi.tator_openapi.models.membership import Membership
from tator.openapi.tator_openapi.models.membership_spec import MembershipSpec
from tator.openapi.tator_openapi.models.membership_update import MembershipUpdate
from tator.openapi.tator_openapi.models.message_response import MessageResponse
from tator.openapi.tator_openapi.models.multi_definition import MultiDefinition
from tator.openapi.tator_openapi.models.not_found_response import NotFoundResponse
from tator.openapi.tator_openapi.models.notify_spec import NotifySpec
from tator.openapi.tator_openapi.models.organization import Organization
from tator.openapi.tator_openapi.models.organization_spec import OrganizationSpec
from tator.openapi.tator_openapi.models.organization_update import OrganizationUpdate
from tator.openapi.tator_openapi.models.parameter import Parameter
from tator.openapi.tator_openapi.models.password_reset_spec import PasswordResetSpec
from tator.openapi.tator_openapi.models.project import Project
from tator.openapi.tator_openapi.models.project_spec import ProjectSpec
from tator.openapi.tator_openapi.models.project_update import ProjectUpdate
from tator.openapi.tator_openapi.models.resolution_config import ResolutionConfig
from tator.openapi.tator_openapi.models.row_protection import RowProtection
from tator.openapi.tator_openapi.models.row_protection_spec import RowProtectionSpec
from tator.openapi.tator_openapi.models.row_protection_update_spec import RowProtectionUpdateSpec
from tator.openapi.tator_openapi.models.s3_storage_config import S3StorageConfig
from tator.openapi.tator_openapi.models.section import Section
from tator.openapi.tator_openapi.models.section_bulk_update import SectionBulkUpdate
from tator.openapi.tator_openapi.models.section_bulk_update_path_substitution import SectionBulkUpdatePathSubstitution
from tator.openapi.tator_openapi.models.section_spec import SectionSpec
from tator.openapi.tator_openapi.models.section_update import SectionUpdate
from tator.openapi.tator_openapi.models.state import State
from tator.openapi.tator_openapi.models.state_bulk_delete import StateBulkDelete
from tator.openapi.tator_openapi.models.state_bulk_update import StateBulkUpdate
from tator.openapi.tator_openapi.models.state_delete import StateDelete
from tator.openapi.tator_openapi.models.state_id_query import StateIdQuery
from tator.openapi.tator_openapi.models.state_merge_update import StateMergeUpdate
from tator.openapi.tator_openapi.models.state_spec import StateSpec
from tator.openapi.tator_openapi.models.state_trim_update import StateTrimUpdate
from tator.openapi.tator_openapi.models.state_type import StateType
from tator.openapi.tator_openapi.models.state_type_spec import StateTypeSpec
from tator.openapi.tator_openapi.models.state_type_update import StateTypeUpdate
from tator.openapi.tator_openapi.models.state_update import StateUpdate
from tator.openapi.tator_openapi.models.temporary_file import TemporaryFile
from tator.openapi.tator_openapi.models.temporary_file_spec import TemporaryFileSpec
from tator.openapi.tator_openapi.models.token import Token
from tator.openapi.tator_openapi.models.transcode import Transcode
from tator.openapi.tator_openapi.models.transcode_spec import TranscodeSpec
from tator.openapi.tator_openapi.models.upload_completion_spec import UploadCompletionSpec
from tator.openapi.tator_openapi.models.upload_info import UploadInfo
from tator.openapi.tator_openapi.models.upload_part import UploadPart
from tator.openapi.tator_openapi.models.user import User
from tator.openapi.tator_openapi.models.user_spec import UserSpec
from tator.openapi.tator_openapi.models.user_update import UserUpdate
from tator.openapi.tator_openapi.models.version import Version
from tator.openapi.tator_openapi.models.version_spec import VersionSpec
from tator.openapi.tator_openapi.models.version_update import VersionUpdate
from tator.openapi.tator_openapi.models.video_clip import VideoClip
from tator.openapi.tator_openapi.models.video_definition import VideoDefinition




from time import sleep
from uuid import uuid4
import random
from datetime import datetime
import pytest

import tator


def random_localization(project, box_type, video_obj, post=False):
    x = random.uniform(0.0, 1.0)
    y = random.uniform(0.0, 1.0)
    w = random.uniform(0.0, 1.0 - x)
    h = random.uniform(0.0, 1.0 - y)
    attributes = {
        "test_bool": random.choice([False, True]),
        "test_int": random.randint(-1000, 1000),
        "test_float": random.uniform(-1000.0, 1000.0),
        "test_enum": random.choice(["a", "b", "c"]),
        "test_string": str(uuid4()),
        "test_datetime": datetime.now().isoformat(),
        "test_geopos": [random.uniform(-180.0, 180.0), random.uniform(-90.0, 90.0)],
        "test_float_array": [random.uniform(-1.0, 1.0) for _ in range(3)],
    }
    out = {
        "x": x,
        "y": y,
        "width": w,
        "height": h,
        "project": project,
        "type": box_type,
        "media_id": video_obj.id,
        "frame": random.randint(0, video_obj.num_frames - 1),
        "attributes": attributes
    }

    return {**out}


allowed_mutations = {
    'bool': [],
    'int': [ 'float'],
    'float': ['int'],
    'enum': ['string'],
    'string': ['enum', ],
    'datetime': [],
    'geopos': [],
    'float_array': [],
}


def mutation_helper(tator_api, type_getter, type_id, params):
    uid = f" {uuid4()}"
    source_name = params["source_name"]
    dest_name = params.get("dest_name", source_name)
    source_name += uid
    dest_name += uid
    source_dtype = params["source_dtype"]
    dest_dtype = params.get("dest_dtype")
    assert_cnt = 0

    entity_type = type_getter(type_id)
    addition = {
        "entity_type": f"{type(entity_type).__name__}",
        "addition": {"name": source_name, "dtype": source_dtype},
    }
    if source_dtype == "enum":
        addition["addition"]["choices"] = ["a", "b", "c"]
    if source_dtype == "float_array":
        addition["addition"]["size"] = 3
    tator_api.create_attribute_type(id=type_id, attribute_type_spec=addition)
    entity_type = type_getter(type_id)

    # Check attribute name before changing it
    assert any(attr.name == source_name for attr in entity_type.attribute_types)
    assert_cnt += 1

    for attr in entity_type.attribute_types:
        if attr.name == source_name:
            # Check the `dtype` before changing it
            assert attr.dtype == source_dtype
            assert_cnt += 1
            break
    else:
        # If the new attribute is not found above, we should fail
        assert False

    mutation = {
        "entity_type": addition["entity_type"],
        "current_name": source_name,
        "attribute_type_update": {"name": dest_name},
    }
    if dest_dtype:
        mutation["attribute_type_update"]["dtype"] = dest_dtype

        if dest_dtype == "enum":
            mutation["attribute_type_update"]["choices"] = ["a", "b", "c"]
        if dest_dtype == "float_array":
            mutation["attribute_type_update"]["size"] = 3

    # A type mutation not in the list of allowed mutations will raise an `ApiException` and the
    # expected `dtype` will be the same as `source_dtype`
    if dest_dtype is None or dest_dtype in allowed_mutations[source_dtype] or dest_dtype == source_dtype:
        expected_dtype = dest_dtype if dest_dtype else source_dtype
        expected_name = dest_name
        tator_api.update_attribute_type(id=type_id, attribute_type_update=mutation)
    else:
        expected_dtype = source_dtype
        expected_name = source_name
        with pytest.raises(tator.openapi.tator_openapi.exceptions.ApiException) as excinfo:
            tator_api.update_attribute_type(id=type_id, attribute_type_update=mutation)

    entity_type = type_getter(type_id)

    # Check new attribute name
    assert any(attr.name == expected_name for attr in entity_type.attribute_types)

    # No attributes should have the old name, if we are expecting a change
    if source_name != expected_name:
        assert all(attr.name != source_name for attr in entity_type.attribute_types)

    for attr in entity_type.attribute_types:
        if attr.name == expected_name:
            # Check that the `dtype` has been updated
            assert attr.dtype == expected_dtype
            break
    else:
        # If the new attribute is not found above, we should fail
        assert False

    # Delete the attribute to keep clean for other tests
    attribute_delete = {
        "entity_type": addition["entity_type"],
        "name": expected_name,
    }
    tator_api.delete_attribute_type(id=type_id, attribute_type_delete=attribute_delete)


@pytest.mark.parametrize("source_dtype", allowed_mutations.keys())
@pytest.mark.parametrize("dest_dtype", allowed_mutations.keys())
def test_box_type_dtype_change(host, token, project, attribute_box_type, source_dtype, dest_dtype):
    tator_api = tator.get_api(host, token)
    params = {
        "source_name": f"{source_dtype} attribute to modify {uuid4()}",
        "source_dtype": source_dtype,
        "dest_dtype": dest_dtype,
        "expected_asserts": 4,
    }
    mutation_helper(tator_api, tator_api.get_localization_type, attribute_box_type, params)


@pytest.mark.parametrize("dtype", allowed_mutations.keys())
def test_box_type_invalid_dtype_change(host, token, project, attribute_box_type, dtype):
    tator_api = tator.get_api(host, token)
    params = {
        "source_name": f"{dtype} attribute to modify {uuid4()}",
        "source_dtype": dtype,
        "dest_dtype": "unknown",
        "expected_asserts": 4,
    }
    mutation_helper(tator_api, tator_api.get_localization_type, attribute_box_type, params)


@pytest.mark.parametrize("dtype", allowed_mutations.keys())
def test_box_type_name_change(host, token, project, attribute_box_type, dtype):
    tator_api = tator.get_api(host, token)
    params = {
        "source_name": f"{dtype} attribute to rename {uuid4()}",
        "source_dtype": dtype,
        "dest_name": f"renamed {dtype} attribute",
        "expected_asserts": 5,
    }
    mutation_helper(tator_api, tator_api.get_localization_type, attribute_box_type, params)


@pytest.mark.parametrize("source_dtype", allowed_mutations.keys())
@pytest.mark.parametrize("dest_dtype", allowed_mutations.keys())
def test_box_type_full_mutation(host, token, project, attribute_box_type, source_dtype, dest_dtype):
    params = {
        "source_name": f"{source_dtype} attribute to modify {uuid4()}",
        "source_dtype": source_dtype,
        "dest_name": f"renamed {source_dtype} -> {dest_dtype} attribute",
        "dest_dtype": dest_dtype,
        "expected_asserts": 5 if dest_dtype in allowed_mutations[source_dtype] else 4,
    }
    tator_api = tator.get_api(host, token)
    mutation_helper(tator_api, tator_api.get_localization_type, attribute_box_type, params)


@pytest.mark.parametrize("dtype", allowed_mutations.keys())
def test_video_and_image_type_name_change(
    host, token, project, attribute_video_type, image_type, dtype
):
    uid = f"{uuid4()}"
    source_name = f"{dtype} attribute to rename {uid}"
    dest_name = f"renamed {dtype} attribute {uid}"
    source_dtype = dtype
    tator_api = tator.get_api(host, token)

    # Add attribute to attribute_video_type
    entity_type = tator_api.get_media_type(attribute_video_type)
    addition = {
        "entity_type": "MediaType",
        "addition": {"name": source_name, "dtype": source_dtype},
    }
    if source_dtype == "enum":
        addition["addition"]["choices"] = ["a", "b", "c"]
    if source_dtype == "float_array":
        addition["addition"]["size"] = 3
    tator_api.create_attribute_type(id=attribute_video_type, attribute_type_spec=addition)
    entity_type = tator_api.get_media_type(attribute_video_type)

    # Check attribute name before changing it
    assert any(attr.name == source_name for attr in entity_type.attribute_types)

    # Add same attribute to image_type
    entity_type = tator_api.get_media_type(image_type)
    addition = {
        "entity_type": "MediaType",
        "addition": {"name": source_name, "dtype": source_dtype},
    }
    if source_dtype == "enum":
        addition["addition"]["choices"] = ["a", "b", "c"]
    if source_dtype == "float_array":
        addition["addition"]["size"] = 3
    tator_api.create_attribute_type(id=image_type, attribute_type_spec=addition)
    entity_type = tator_api.get_media_type(image_type)

    # Check attribute name before changing it
    assert any(attr.name == source_name for attr in entity_type.attribute_types)

    # Mutate attribute on attribute_video_type
    mutation = {
        "entity_type": "MediaType",
        "current_name": source_name,
        "attribute_type_update": {"name": dest_name, "dtype": source_dtype},
    }
    if source_dtype == "enum":
        mutation["attribute_type_update"]["choices"] = ["a", "b", "c"]
    if source_dtype == "float_array":
        mutation["attribute_type_update"]["size"] = 3

    tator_api.update_attribute_type(id=attribute_video_type, attribute_type_update=mutation)

    entity_type = tator_api.get_media_type(attribute_video_type)

    # Check new attribute name
    assert any(attr.name == dest_name for attr in entity_type.attribute_types)

    # No attributes should have the old name
    assert all(attr.name != source_name for attr in entity_type.attribute_types)

    # Update the image type too
    tator_api.update_attribute_type(id=image_type, attribute_type_update=mutation)
    entity_type = tator_api.get_media_type(image_type)
    assert any(attr.name == dest_name for attr in entity_type.attribute_types)
    assert all(attr.name != source_name for attr in entity_type.attribute_types)

    # Delete the attribute to keep clean for other tests
    attribute_delete = {
        "entity_type": "MediaType",
        "name": dest_name,
    }
    tator_api.delete_attribute_type(id=attribute_video_type, attribute_type_delete=attribute_delete)
    tator_api.delete_attribute_type(id=image_type, attribute_type_delete=attribute_delete)

def test_box_type_attribute_mutation_fail(
        host, token, project, attribute_video, attribute_box_type
):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(attribute_video)

    # Create localizations, but less than 100
    num_localizations = 75
    boxes = [
        random_localization(project, attribute_box_type, video_obj, post=True)
        for _ in range(num_localizations)
    ]
    box_ids = [
        box_id
        for response in tator.util.chunked_create(
            tator_api.create_localization_list, project, body=boxes
        )
        for box_id in response.id
    ]

    assert len(box_ids) == len(boxes)

    # Make sure the new attribute does not exist already
    value = str(uuid4()).lower()
    new_attr_name = f"New int {value}"
    entity_type = tator_api.get_localization_type(attribute_box_type)
    assert all(attr.name != new_attr_name for attr in entity_type.attribute_types)
    addition = {
        "entity_type": "LocalizationType",
        "addition": {"name": new_attr_name, "dtype": "int", "default": 0},
    }
    tator_api.create_attribute_type(id=attribute_box_type, attribute_type_spec=addition)
    entity_type = tator_api.get_localization_type(attribute_box_type)

    # Check for added attribute
    assert any(attr.name == new_attr_name for attr in entity_type.attribute_types)

    mutation = {
        "entity_type": "LocalizationType",
        "current_name": new_attr_name,
        "attribute_type_update": {"name": new_attr_name, "dtype": "float"},
    }
    tator_api.update_attribute_type(id=attribute_box_type, attribute_type_update=mutation)

    # Check for mutated attribute
    entity_type = tator_api.get_localization_type(attribute_box_type)
    for attr in entity_type.attribute_types:
        if attr.name == new_attr_name:
            assert attr.dtype == "float"
            break
    else:
        assert False, f"Attribute {new_attr_name} not found"

    # Clean up
    params = {"media_id": [attribute_video], "type": attribute_box_type}
    tator_api.delete_localization_list(project, **params)

    # Delete the attribute to keep clean for other tests
    attribute_delete = {
        "entity_type": "LocalizationType",
        "name": new_attr_name,
    }
    tator_api.delete_attribute_type(id=attribute_box_type, attribute_type_delete=attribute_delete)



# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class Announcement(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_datetime': 'str',
        'eol_datetime': 'str',
        'id': 'int',
        'markdown': 'str'
    }

    attribute_map = {
        'created_datetime': 'created_datetime',
        'eol_datetime': 'eol_datetime',
        'id': 'id',
        'markdown': 'markdown'
    }

    def __init__(self, created_datetime=None, eol_datetime=None, id=None, markdown=None, local_vars_configuration=None):  # noqa: E501
        """Announcement - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._created_datetime = None
        self._eol_datetime = None
        self._id = None
        self._markdown = None
        self.discriminator = None

        if created_datetime is not None:
            self.created_datetime = created_datetime
        if eol_datetime is not None:
            self.eol_datetime = eol_datetime
        if id is not None:
            self.id = id
        if markdown is not None:
            self.markdown = markdown

    @property
    def created_datetime(self):
        """
        When the announcement was made.

        :return: The created_datetime of this Announcement. 
        :rtype: str
        """
        return self._created_datetime

    @created_datetime.setter
    def created_datetime(self, created_datetime):
        """
        When the announcement was made.

        :param created_datetime: The created_datetime of this Announcement.
        :type: str
        """

        self._created_datetime = created_datetime

    @property
    def eol_datetime(self):
        """
        When the announcement will expire.

        :return: The eol_datetime of this Announcement. 
        :rtype: str
        """
        return self._eol_datetime

    @eol_datetime.setter
    def eol_datetime(self, eol_datetime):
        """
        When the announcement will expire.

        :param eol_datetime: The eol_datetime of this Announcement.
        :type: str
        """

        self._eol_datetime = eol_datetime

    @property
    def id(self):
        """
        Unique integer identifying an announcement.

        :return: The id of this Announcement. 
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique integer identifying an announcement.

        :param id: The id of this Announcement.
        :type: int
        """

        self._id = id

    @property
    def markdown(self):
        """
        Markdown formatted contents of the announcement.

        :return: The markdown of this Announcement. 
        :rtype: str
        """
        return self._markdown

    @markdown.setter
    def markdown(self, markdown):
        """
        Markdown formatted contents of the announcement.

        :param markdown: The markdown of this Announcement.
        :type: str
        """

        self._markdown = markdown

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Announcement):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Announcement):
            return True

        return self.to_dict() != other.to_dict()



import os
import math

from ._download_file import _download_file

def _is_none_or_eq(variable, match):
    """  :returns: True if `variable` is None or equals `match` """
    return variable is None or variable == match

def download_attachment(api, media, out_path, index=0):
    """ Download an attachment from Tator to an off-line location.

    Example:

    .. code-block:: python

        api = tator.get_api(host, token)
        media = api.get_media(media_id)
        out_path = f'/tmp/{media.name}'
        for progress in tator.util.download_attachment(api, media, out_path):
            print(f"Download progress: {progress}%")

        #download second attachment
        for progress in tator.util.download_attachment(api, media, out_path, index=1):
            print(f"Download progress: {progress}%")

    :param api: :class:`tator.TatorApi` object.
    :param media: :class:`tator.Media` object.
    :param path-like out_path: Path to where to download.
    :param index: Integer indicating which attachment to download.
    :returns: Generator that yields progress (0-100).
    """
    if media.media_files is not None:
        attachment = media.media_files.attachment
        if (attachment is None) or (len(attachment) <= index):
            raise ValueError(f"Media {media.id} does not have an attachment at index {index}!")
        else:
            url = attachment[index].path
    return _download_file(api, media.project, url, out_path)




import datetime
import random
import uuid
import time

from PIL import Image
import numpy as np

import tator

def _ints_almost_equal(
    a: int,
    b: int,
    tolerance=1) -> bool:
    """ Returns true if the integers are almost equal (applying tolerance)

    Args:
        a, b: integers to compare (e.g. pixel)
        tolerance: integer tolerance to add for almost equal

    Returns:
        True if almost equal within the provided tolerance. False otherwise.

    """

    return abs(a - b) <= tolerance

def _random_localization(
    project: int,
    media_id: int,
    dtype: str,
    type_id: int,
    frame: int) -> dict:
    ''' Creates a dicctionary defining a randomized localization based on given parameters

    Args:
        project: Unique project ID
        media_id: Unique media ID
        dtype: 'box' or 'dot' or 'line'
        type_id: Unique ID associated with the given localization type
        frame: Frame number associated with the localization

    Returns:
        Dictionary version of the localization object

    '''

    x = random.uniform(0.0, 1.0)
    y = random.uniform(0.0, 1.0)

    if dtype == 'box':
        width = random.uniform(0.0, 1.0 - x)
        height = random.uniform(0.0, 1.0 - y)
        u = 0.0
        v = 0.0

    elif dtype == 'line':
        width = 0.0
        height = 0.0
        u = random.uniform(0.0, 1.0)
        v = random.uniform(0.0, 1.0)

    elif dtype == 'dot':
        width = 0.0
        height = 0.0
        u = 0.0
        v = 0.0

    else:
        raise ValueError("Invalid dtype provided")

    datum = {
        'x': x,
        'y': y,
        'width': width,
        'height': height,
        'u': u,
        'v': v,
        'project': project,
        'media_id': media_id,
        'type': type_id,
        'frame': frame,
    }

    attributes = {
        'test_bool': random.choice([False, True]),
        'test_int': random.randint(-1000, 1000),
        'test_float': random.uniform(-1000.0, 1000.0),
        'test_enum': random.choice(['a', 'b', 'c']),
        'test_string': str(uuid.uuid1()),
        'test_datetime': datetime.datetime.now().isoformat(),
        'test_geopos': [random.uniform(-180.0, 180.0), random.uniform(-90.0, 90.0)],
        'test_float_array': [random.uniform(-1.0, 1.0) for _ in range(3)],
    }

    datum = {**datum, **attributes}

    return datum

def _generate_truth_info(
        localization_datum: dict,
        media_width: int,
        media_height: int,
        dtype: str,
        margins: tuple=None) -> tuple:
    """ Returns dictionary of truth information used for unit test comparisons

    Args:
        localization_datum: Dictionary schema defining a localization
        media_width: Width of media in pixels
        media_heieght: Height of media in pixels
        dtype: 'box', 'line', or 'dot' defining the localization types
        margins: Tuple where [0] is width (pixels), [1] is height(pixels). If none, then ignored.

    Returns:
        Tuple where [0] is width (pixels), [1] is height (pixels)
    """

    MINIMUM_PIXEL_SIZE = 3

    # Define the truth widths and heights (without margins) in pixel space
    if dtype == 'box':
        truth_width = localization_datum['width'] * media_width
        truth_height = localization_datum['height'] * media_height
        default_margins = (0, 0)

    elif dtype == 'line':
        point_a = (localization_datum['x'], localization_datum['y'])
        point_b = (point_a[0] + localization_datum['u'], point_a[1] + localization_datum['v'])
        truth_width = abs(point_a[0] - point_b[0]) * media_width
        truth_height = abs(point_a[1] - point_b[1]) * media_height
        default_margins = (10, 10)

    else: # Dot
        truth_width = 1
        truth_height = 1
        default_margins = (10, 10)

    # Apply specific margins if provided. Otherwise use default.
    if margins is not None:
        truth_width += margins[0] * 2
        truth_height += margins[1] * 2

    else:
        truth_width += default_margins[0] * 2
        truth_height += default_margins[1] * 2

    # Constrain to media size and expected minimum size
    if truth_width > media_width:
        truth_width = media_width
    elif truth_width <= MINIMUM_PIXEL_SIZE:
        truth_width = MINIMUM_PIXEL_SIZE

    if truth_height > media_height:
        truth_height = media_height
    elif truth_height <= MINIMUM_PIXEL_SIZE:
        truth_height = MINIMUM_PIXEL_SIZE

    return round(truth_width), round(truth_height)

def _perform_specific_box_size_test(
        host: str,
        token: str,
        project: int,
        media_id: int,
        box_type: int,
        x: float,
        y: float,
        width: int,
        height: int,
        margins: tuple=None) -> None:
    """ Performs testing with the given parameters using a specific localization box size

    Single test performed. It's based on figuring out the truth width/height and requesting
    the graphic associated with the localization. The truth width/height is then
    compared against the image returned.

    There is a tolerance applied due to floating point rounding going from
    relative to absolute dimensions.

    Args:
        host: Project URL
        token: User token used for connecting to the host
        project: Unique identifier of test project
        media_id: Unique identifier of media associated with the localization
        box_type: Unique identifier of the test box localization type
        x: X value of the box localization (relative)
        y: Y value of the box localization (relative)
        width: Width of the box localization (pixels)
        height: Height of the box localization (pixels)
        margins: Width/height of the margins to apply (pixels). None if should be ignored.
    """

    # Create interface
    tator_api = tator.get_api(host, token)
    media = tator_api.get_media(media_id)

    # Create the localizations and send it over to the server
    if media.num_frames is not None:
        FRAME_NUMBER = random.randint(0, media.num_frames - 1)
    else:
        FRAME_NUMBER = 0

    # Create localization
    datum = {
        'x': x,
        'y': y,
        'width': width / media.width,
        'height': height / media.width,
        'u': 0.0,
        'v': 0.0,
        'project': project,
        'media_id': media_id,
        'type': box_type,
        'frame': FRAME_NUMBER,
    }

    attributes = {
        'test_bool': random.choice([False, True]),
        'test_int': random.randint(-1000, 1000),
        'test_float': random.uniform(-1000.0, 1000.0),
        'test_enum': random.choice(['a', 'b', 'c']),
        'test_string': str(uuid.uuid1()),
        'test_datetime': datetime.datetime.now().isoformat(),
        'test_geopos': [random.uniform(-180.0, 180.0), random.uniform(-90.0, 90.0)],
        'test_float_array': [random.uniform(-1.0, 1.0) for _ in range(3)],
    }

    datum = {**datum, 'attributes': {**attributes}}
    response = tator_api.create_localization_list(project, body=datum)

    # Generate the truth information for unit testing
    truth_graphic_info = {}
    for localization_id in response.id:
        truth_graphic_info[localization_id] = _generate_truth_info(
            localization_datum=datum,
            media_width=media.width,
            media_height=media.height,
            dtype='box',
            margins=margins)

    # Cycle through the localizations, create the thumbnail and compare the image sizes
    for localization_id in truth_graphic_info:

        truth_width = truth_graphic_info[localization_id][0]
        truth_height = truth_graphic_info[localization_id][1]

        if margins is None:
            image_path = tator_api.get_localization_graphic(localization_id)
        else:
            image_path = tator_api.get_localization_graphic(
                localization_id,
                use_default_margins=False,
                margin_x=margins[0],
                margin_y=margins[1])

        img = Image.open(image_path)
        assert _ints_almost_equal(img.size[0], truth_width)
        assert _ints_almost_equal(img.size[1], truth_height)

def _perform_random_sizes_test(
        host: str,
        token: str,
        project: int,
        media_id: int,
        dot_type: int,
        line_type: int,
        box_type: int,
        margins: tuple=None,
        image_size: tuple=None) -> None:
    """ Performs testing with localizations of different types/sizes

    Tests are based on figuring out the truth width/height and requesting
    the graphic associated with each localization. The truth width/height is then
    compared against the image returned.

    There is a tolerance applied due to floating point rounding going from
    relative to absolute dimensions.

    Args:
        host: Project URL
        token: User token used for connecting to the host
        project: Unique identifier of test project
        media_id: Unique identifier of media associated with the localization
        dot_type: Unique identifier of the test dot localization type
        line_type: Unique identifier of the test line localization type
        box_type: Unique identifier of the test box localization type
        margins: Width/height of the margins to apply (pixels). None if should be ignored.
        image_size: Width/height of the image size forced on the localization graphic.
    """

    # Define mapping from type unique ID and dtype string
    LOCALIZATION_TYPES_INT_TO_DTYPE = {
        dot_type: 'dot',
        line_type: 'line',
        box_type: 'box'}

    # Create interface
    tator_api = tator.get_api(host, token)
    media = tator_api.get_media(media_id)

    # Create the localizations and send it over to the server
    if media.num_frames is not None:
        FRAME_NUMBER = random.randint(0, media.num_frames - 1)
    else:
        FRAME_NUMBER = 0

    NUM_LOCALIZATIONS = 10
    localization_list = []
    for localization_type in LOCALIZATION_TYPES_INT_TO_DTYPE:
        for idx in range(NUM_LOCALIZATIONS):

            localization = _random_localization(
                project=project,
                media_id=media_id,
                dtype=LOCALIZATION_TYPES_INT_TO_DTYPE[localization_type],
                type_id=localization_type,
                frame=FRAME_NUMBER)

            localization_list.append(localization)

    response = tator_api.create_localization_list(project, body=localization_list)

    # Generate the truth information for unit testing
    truth_graphic_info = {}
    for localization_id, localization in zip(response.id, localization_list):
        truth_graphic_info[localization_id] = _generate_truth_info(
            localization_datum=localization,
            media_width=media.width,
            media_height=media.height,
            dtype=LOCALIZATION_TYPES_INT_TO_DTYPE[localization['type']],
            margins=margins)

    # Cycle through the localizations, create the thumbnail and compare the image sizes
    for localization_id in truth_graphic_info:

        truth_width = truth_graphic_info[localization_id][0]
        truth_height = truth_graphic_info[localization_id][1]

        if margins is None:
            if image_size is None:
                image_path = tator_api.get_localization_graphic(localization_id)
            else:
                image_size_str = f'{image_size[0]}x{image_size[1]}'
                image_path = tator_api.get_localization_graphic(localization_id, force_scale=image_size_str)
        else:
            if image_size is None:
                image_path = tator_api.get_localization_graphic(
                    localization_id,
                    use_default_margins=False,
                    margin_x=margins[0],
                    margin_y=margins[1])
            else:
                image_size_str = f'{image_size[0]}x{image_size[1]}'
                image_path = tator_api.get_localization_graphic(
                    localization_id,
                    use_default_margins=False,
                    margin_x=margins[0],
                    margin_y=margins[1],
                    force_scale=image_size_str)

        img = Image.open(image_path)

        if image_size is not None:
            truth_width = image_size[0]
            truth_height = image_size[1]

        assert _ints_almost_equal(img.size[0], truth_width)
        assert _ints_almost_equal(img.size[1], truth_height)

def test_localization_graphic_image(
        host: str,
        token: str,
        project: int,
        image: int,
        dot_type: int,
        line_type: int,
        box_type: int) -> None:
    """ Test localization graphic endpoint with an image

    Args:
        host: Project URL
        token: User token used for connecting to the host
        project: Unique identifier of test project
        media_id: Unique identifier of media associated with the localization
        dot_type: Unique identifier of the test dot localization type
        line_type: Unique identifier of the test line localization type
        box_type: Unique identifier of the test box localization type

    Create a variety of different localizations
    Create thumbnails from each of them using default margins
    Create thumbnails from each of them using explicit margins
    Create thumbnails from localizations that have a small width/height
    Create thumbnail from localization with margins that expand the media's dimensions

    """

    _perform_random_sizes_test(
        host=host,
        token=token,
        project=project,
        media_id=image,
        dot_type=dot_type,
        line_type=line_type,
        box_type=box_type)

    _perform_random_sizes_test(
        host=host,
        token=token,
        project=project,
        media_id=image,
        dot_type=dot_type,
        line_type=line_type,
        box_type=box_type,
        margins=(100, 100))

    _perform_random_sizes_test(
        host=host,
        token=token,
        project=project,
        media_id=image,
        dot_type=dot_type,
        line_type=line_type,
        box_type=box_type,
        margins=(10, 10),
        image_size=(50, 50))

    _perform_specific_box_size_test(
        host=host,
        token=token,
        project=project,
        media_id=image,
        box_type=box_type,
        x=0.5,
        y=0.5,
        width=1,
        height=1)

    _perform_specific_box_size_test(
        host=host,
        token=token,
        project=project,
        media_id=image,
        box_type=box_type,
        x=0.5,
        y=0.5,
        width=1,
        height=10)

    _perform_specific_box_size_test(
        host=host,
        token=token,
        project=project,
        media_id=image,
        box_type=box_type,
        x=0.5,
        y=0.5,
        width=10,
        height=1)

    _perform_specific_box_size_test(
        host=host,
        token=token,
        project=project,
        media_id=image,
        box_type=box_type,
        x=0.0,
        y=0.0,
        width=50,
        height=50,
        margins=(10000,10000))

def test_localization_graphic_video(
        host: str,
        token: str,
        project: int,
        video: int,
        dot_type: int,
        line_type: int,
        box_type: int) -> None:
    """ Test localization graphic endpoint with a video

    Args:
        host: Project URL
        token: User token used for connecting to the host
        project: Unique identifier of test project
        media_id: Unique identifier of media associated with the localization
        dot_type: Unique identifier of the test dot localization type
        line_type: Unique identifier of the test line localization type
        box_type: Unique identifier of the test box localization type

    Create a variety of different localizations over a few different frames
    Create thumbnails from each of them using default marings
    Create thumbnails from each of them using explicit marings
    Create thumbnails from localizations that have a small width/height
    Create thumbnail from localization with margins that expand the media's dimensions

    """

    _perform_random_sizes_test(
        host=host,
        token=token,
        project=project,
        media_id=video,
        dot_type=dot_type,
        line_type=line_type,
        box_type=box_type)

    _perform_random_sizes_test(
        host=host,
        token=token,
        project=project,
        media_id=video,
        dot_type=dot_type,
        line_type=line_type,
        box_type=box_type,
        margins=(100, 100))

    _perform_random_sizes_test(
        host=host,
        token=token,
        project=project,
        media_id=video,
        dot_type=dot_type,
        line_type=line_type,
        box_type=box_type,
        margins=(10, 10),
        image_size=(50, 50))

    _perform_specific_box_size_test(
        host=host,
        token=token,
        project=project,
        media_id=video,
        box_type=box_type,
        x=0.5,
        y=0.5,
        width=1,
        height=1)

    _perform_specific_box_size_test(
        host=host,
        token=token,
        project=project,
        media_id=video,
        box_type=box_type,
        x=0.5,
        y=0.5,
        width=1,
        height=10)

    _perform_specific_box_size_test(
        host=host,
        token=token,
        project=project,
        media_id=video,
        box_type=box_type,
        x=0.5,
        y=0.5,
        width=10,
        height=1)

    _perform_specific_box_size_test(
        host=host,
        token=token,
        project=project,
        media_id=video,
        box_type=box_type,
        x=0.0,
        y=0.0,
        width=50,
        height=50,
        margins=(10000,10000))



# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class Section(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'attributes': 'dict(str, object)',
        'created_by': 'int',
        'created_datetime': 'datetime',
        'dtype': 'str',
        'elemental_id': 'str',
        'explicit_listing': 'bool',
        'id': 'int',
        'master_section': 'int',
        'media': 'list[int]',
        'name': 'str',
        'object_search': 'AttributeOperationSpec',
        'path': 'str',
        'project': 'int',
        'related_search': 'AttributeOperationSpec',
        'tator_user_sections': 'str',
        'visible': 'bool'
    }

    attribute_map = {
        'attributes': 'attributes',
        'created_by': 'created_by',
        'created_datetime': 'created_datetime',
        'dtype': 'dtype',
        'elemental_id': 'elemental_id',
        'explicit_listing': 'explicit_listing',
        'id': 'id',
        'master_section': 'master_section',
        'media': 'media',
        'name': 'name',
        'object_search': 'object_search',
        'path': 'path',
        'project': 'project',
        'related_search': 'related_search',
        'tator_user_sections': 'tator_user_sections',
        'visible': 'visible'
    }

    def __init__(self, attributes=None, created_by=None, created_datetime=None, dtype=None, elemental_id=None, explicit_listing=None, id=None, master_section=None, media=None, name=None, object_search=None, path=None, project=None, related_search=None, tator_user_sections=None, visible=None, local_vars_configuration=None):  # noqa: E501
        """Section - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._attributes = None
        self._created_by = None
        self._created_datetime = None
        self._dtype = None
        self._elemental_id = None
        self._explicit_listing = None
        self._id = None
        self._master_section = None
        self._media = None
        self._name = None
        self._object_search = None
        self._path = None
        self._project = None
        self._related_search = None
        self._tator_user_sections = None
        self._visible = None
        self.discriminator = None

        if attributes is not None:
            self.attributes = attributes
        if created_by is not None:
            self.created_by = created_by
        self.created_datetime = created_datetime
        if dtype is not None:
            self.dtype = dtype
        self.elemental_id = elemental_id
        if explicit_listing is not None:
            self.explicit_listing = explicit_listing
        if id is not None:
            self.id = id
        if master_section is not None:
            self.master_section = master_section
        if media is not None:
            self.media = media
        if name is not None:
            self.name = name
        if object_search is not None:
            self.object_search = object_search
        self.path = path
        if project is not None:
            self.project = project
        if related_search is not None:
            self.related_search = related_search
        if tator_user_sections is not None:
            self.tator_user_sections = tator_user_sections
        if visible is not None:
            self.visible = visible

    @property
    def attributes(self):
        """
        Object containing attribute values.

        :return: The attributes of this Section. 
        :rtype: dict(str, object)
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """
        Object containing attribute values.

        :param attributes: The attributes of this Section.
        :type: dict(str, object)
        """

        self._attributes = attributes

    @property
    def created_by(self):
        """
        Unique integer identifying the user who created this localization.

        :return: The created_by of this Section. 
        :rtype: int
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """
        Unique integer identifying the user who created this localization.

        :param created_by: The created_by of this Section.
        :type: int
        """

        self._created_by = created_by

    @property
    def created_datetime(self):
        """
        The elemental ID of the object.

        :return: The created_datetime of this Section. 
        :rtype: datetime
        """
        return self._created_datetime

    @created_datetime.setter
    def created_datetime(self, created_datetime):
        """
        The elemental ID of the object.

        :param created_datetime: The created_datetime of this Section.
        :type: datetime
        """

        self._created_datetime = created_datetime

    @property
    def dtype(self):
        """
        Type of the section.

        :return: The dtype of this Section. 
        :rtype: str
        """
        return self._dtype

    @dtype.setter
    def dtype(self, dtype):
        """
        Type of the section.

        :param dtype: The dtype of this Section.
        :type: str
        """
        allowed_values = ["folder", "playlist", "saved_search"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and dtype not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `dtype` ({0}), must be one of {1}"  # noqa: E501
                .format(dtype, allowed_values)
            )

        self._dtype = dtype

    @property
    def elemental_id(self):
        """
        The elemental ID of the object.

        :return: The elemental_id of this Section. 
        :rtype: str
        """
        return self._elemental_id

    @elemental_id.setter
    def elemental_id(self, elemental_id):
        """
        The elemental ID of the object.

        :param elemental_id: The elemental_id of this Section.
        :type: str
        """

        self._elemental_id = elemental_id

    @property
    def explicit_listing(self):
        """
        Deprecated, set `dtype` to `playlist` instead. Determines whether the section is explicitly made up of media IDs.

        :return: The explicit_listing of this Section. 
        :rtype: bool
        """
        return self._explicit_listing

    @explicit_listing.setter
    def explicit_listing(self, explicit_listing):
        """
        Deprecated, set `dtype` to `playlist` instead. Determines whether the section is explicitly made up of media IDs.

        :param explicit_listing: The explicit_listing of this Section.
        :type: bool
        """

        self._explicit_listing = explicit_listing

    @property
    def id(self):
        """
        Unique integer identifying the section.

        :return: The id of this Section. 
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique integer identifying the section.

        :param id: The id of this Section.
        :type: int
        """

        self._id = id

    @property
    def master_section(self):
        """
        Unique ID to define the ACL origin of this object. If not set defaults to the master section of the primary_section.

        :return: The master_section of this Section. 
        :rtype: int
        """
        return self._master_section

    @master_section.setter
    def master_section(self, master_section):
        """
        Unique ID to define the ACL origin of this object. If not set defaults to the master section of the primary_section.

        :param master_section: The master_section of this Section.
        :type: int
        """

        self._master_section = master_section

    @property
    def media(self):
        """
        List of media IDs that belong in this section.

        :return: The media of this Section. 
        :rtype: list[int]
        """
        return self._media

    @media.setter
    def media(self, media):
        """
        List of media IDs that belong in this section.

        :param media: The media of this Section.
        :type: list[int]
        """

        self._media = media

    @property
    def name(self):
        """
        Unique name of the section.

        :return: The name of this Section. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Unique name of the section.

        :param name: The name of this Section.
        :type: str
        """

        self._name = name

    @property
    def object_search(self):
        """

        :return: The object_search of this Section. 
        :rtype: AttributeOperationSpec
        """
        return self._object_search

    @object_search.setter
    def object_search(self, object_search):
        """

        :param object_search: The object_search of this Section.
        :type: AttributeOperationSpec
        """

        self._object_search = object_search

    @property
    def path(self):
        """
        A path to represent nested sections. If not supplied, defaults to `re.sub(r'[^A-Za-z0-9_-]',path)`

        :return: The path of this Section. 
        :rtype: str
        """
        return self._path

    @path.setter
    def path(self, path):
        """
        A path to represent nested sections. If not supplied, defaults to `re.sub(r'[^A-Za-z0-9_-]',path)`

        :param path: The path of this Section.
        :type: str
        """

        self._path = path

    @property
    def project(self):
        """
        Unique integer identifying the project associated with the section.

        :return: The project of this Section. 
        :rtype: int
        """
        return self._project

    @project.setter
    def project(self, project):
        """
        Unique integer identifying the project associated with the section.

        :param project: The project of this Section.
        :type: int
        """

        self._project = project

    @property
    def related_search(self):
        """

        :return: The related_search of this Section. 
        :rtype: AttributeOperationSpec
        """
        return self._related_search

    @related_search.setter
    def related_search(self, related_search):
        """

        :param related_search: The related_search of this Section.
        :type: AttributeOperationSpec
        """

        self._related_search = related_search

    @property
    def tator_user_sections(self):
        """
        Attribute that is applied to media to identify membership to a section.

        :return: The tator_user_sections of this Section. 
        :rtype: str
        """
        return self._tator_user_sections

    @tator_user_sections.setter
    def tator_user_sections(self, tator_user_sections):
        """
        Attribute that is applied to media to identify membership to a section.

        :param tator_user_sections: The tator_user_sections of this Section.
        :type: str
        """

        self._tator_user_sections = tator_user_sections

    @property
    def visible(self):
        """
        Determines the visibility in the UI.

        :return: The visible of this Section. 
        :rtype: bool
        """
        return self._visible

    @visible.setter
    def visible(self, visible):
        """
        Determines the visibility in the UI.

        :param visible: The visible of this Section.
        :type: bool
        """

        self._visible = visible

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Section):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Section):
            return True

        return self.to_dict() != other.to_dict()



import os
import subprocess
import shutil
import urllib.request

from setuptools import setup, find_packages
import requests
import yaml
import json

REQUIRES = [
    "certifi >= 14.05.14",
    "Pillow >= 10.3.0",
    "progressbar2 >= 3.51.4",
    "psutil >= 5.9.5",
    "python-dateutil >= 2.5.3",
    "pyyaml >= 5.4",
    "requests >= 2.32.4",
    "six >= 1.10",
    "urllib3 >= 2.5.0",
]

SCHEMA_FILENAME = 'schema.yaml'
CONFIG_FILENAME = 'config.json'
OPENAPI_GENERATOR_VERSION = '4.3.1'
OPENAPI_GENERATOR_JAR = f'openapi-generator-cli-{OPENAPI_GENERATOR_VERSION}.jar'
OPENAPI_GENERATOR_URL = f'https://tator-ci.s3.us-east-1.amazonaws.com/{OPENAPI_GENERATOR_JAR}'

def get_version():
    with open(CONFIG_FILENAME, 'r') as f:
        config = json.load(f)
    return config['packageVersion']

def remove_oneof(data):
    """ Removes oneOf key from a dict and recursively calls this function on
        other dict values.
    """
    if 'oneOf' in data:
        del data['oneOf']
    for key in data:
        if isinstance(data[key], dict):
            remove_oneof(data[key])
    return data

class NoAliasDumper(yaml.Dumper):
    def ignore_aliases(self, data):
        return True

def ensure_java():
    """Check if Java is installed, if not, provide instructions."""
    try:
        result = subprocess.run(['java', '-version'], capture_output=True, text=True)
        if result.returncode != 0:
            raise RuntimeError("Java is not installed")
        return True
    except (subprocess.CalledProcessError, FileNotFoundError):
        print("ERROR: Java is required to run OpenAPI Generator")
        print("Please install Java 8 or later:")
        print("  Ubuntu/Debian: sudo apt-get install openjdk-11-jre")
        print("  macOS: brew install openjdk@11")
        print("  Or download from: https://adoptium.net/")
        return False

def download_openapi_generator():
    """Download the OpenAPI Generator JAR if it doesn't exist."""
    if not os.path.exists(OPENAPI_GENERATOR_JAR):
        print(f"Downloading OpenAPI Generator {OPENAPI_GENERATOR_VERSION}...")
        urllib.request.urlretrieve(OPENAPI_GENERATOR_URL, OPENAPI_GENERATOR_JAR)
        print(f"Downloaded {OPENAPI_GENERATOR_JAR}")

def codegen():
    """ Fetches a schema from cloud.tator.io if one does not exist, then 
        use openapi-generator to generate openapi code from it.
    """
    # Check Java is available
    if not ensure_java():
        raise RuntimeError("Java is required but not found")
    
    # Download OpenAPI Generator JAR if needed
    pwd = os.path.dirname(os.path.abspath(__file__))
    jar_path = os.path.join(pwd, OPENAPI_GENERATOR_JAR)
    if not os.path.exists(jar_path):
        print(f"Downloading OpenAPI Generator {OPENAPI_GENERATOR_VERSION}...")
        urllib.request.urlretrieve(OPENAPI_GENERATOR_URL, jar_path)
        print(f"Downloaded {OPENAPI_GENERATOR_JAR}")
    
    # Retrieve schema if it does not exist.
    if not os.path.exists(SCHEMA_FILENAME):
        response = requests.get("https://cloud.tator.io/schema")
        assert response.status_code == 200
        with open(SCHEMA_FILENAME, 'wb') as f:
            f.write(response.content)

    # Remove any oneOf entries from the schema, as they are not handled
    # well by openapi generator.
    with open(SCHEMA_FILENAME, 'r') as f:
        schema = yaml.safe_load(f)
    schema = remove_oneof(schema)
    with open(SCHEMA_FILENAME, 'w') as f:
        yaml.dump(schema, f, Dumper=NoAliasDumper)

    # Get the git SHA ID.
    cmd = ['git', 'rev-parse', 'HEAD']
    git_rev = subprocess.check_output(cmd).strip().decode('utf-8')

    # Generate code using OpenAPI Generator JAR (works natively on all architectures)
    # Use temp directory to avoid permission issues
    import tempfile
    temp_out = tempfile.mkdtemp(prefix='tator-py-')
    out_dir = os.path.join(temp_out, f'tator-py-{git_rev}')
    
    # Ensure output directory exists
    os.makedirs(out_dir, exist_ok=True)
    
    cmd = [
        'java', '-jar', jar_path, 'generate',
        '-c', CONFIG_FILENAME,
        '-i', SCHEMA_FILENAME,
        '-g', 'python',
        '-o', out_dir,
        '-t', os.path.join(pwd, 'templates'),
    ]
    
    import platform
    print(f"Running OpenAPI Generator on {platform.machine()} architecture...")
    subprocess.run(cmd, check=True, cwd=pwd)

    # Remove the schema.
    os.remove(SCHEMA_FILENAME)

    # Copy relevant directories into openapi.
    openapi_dir = os.path.join(pwd, 'tator/openapi')
    os.makedirs(openapi_dir, exist_ok=True)
    for subpath in ['README.md', 'tator_openapi', 'docs']:
        src = os.path.join(out_dir, subpath)
        dst = os.path.join(openapi_dir, subpath)
        if os.path.isfile(src):
            shutil.copy(src, dst)
        else:
            if os.path.exists(dst):
                shutil.rmtree(dst)
            shutil.copytree(src, dst)
    
    # Clean up temp directory
    shutil.rmtree(temp_out)

codegen()
setup(
    name="tator",
    version=get_version(),
    description="Python client for Tator",
    author="CVision AI, Inc.",
    author_email="info@cvisionai.com",
    url="https://github.com/cvisionai/tator-py",
    keywords=["OpenAPI", "OpenAPI-Generator", "Tator"],
    install_requires=REQUIRES,
    packages=find_packages(exclude=["test", "tests"]),
    package_data={"tator": ["tator/util/tator-symbol.png", "tator/util/tator-symbol.gif"]},
    include_package_data=True,
    long_description="""\
    Python client for Tator.
    """
)



import os
import tempfile

import tator

def test_download_image(host, token, image):
    tator_api = tator.get_api(host, token)
    image_obj = tator_api.get_media(image)

    image_files = image_obj.media_files.image
    thumbnail = image_obj.media_files.thumbnail

    assert len(image_files) > 0, "Must have at least 1 image file!"

    available = [x.resolution[0] for x in image_files]
    best = max(available)
    best_idx = available.index(best)

    with tempfile.TemporaryDirectory() as td:
        image_path = os.path.join(td, image_obj.name)
        for progress in tator.download_media(
                tator_api,
                image_obj,
                image_path,
                quality=None,
                media_type=None):
            print(f"Media download progress: {progress}%")
        assert os.path.exists(image_path)
        assert os.stat(image_path).st_size == image_files[best_idx].size

    available = [x.resolution[0] for x in thumbnail]
    best = max(available)
    best_idx = available.index(best)
    with tempfile.TemporaryDirectory() as td:
        image_path = os.path.join(td, image_obj.name)
        for progress in tator.download_media(
                tator_api,
                image_obj,
                image_path,
                quality=None,
                media_type='thumbnail'):
            print(f"Media download progress: {progress}%")
        assert os.path.exists(image_path)
        assert os.stat(image_path).st_size == thumbnail[best_idx].size


    for image_file in image_files:
        with tempfile.TemporaryDirectory() as td:
            image_path = os.path.join(td, image_obj.name)
            for progress in tator.download_media(
                    tator_api,
                    image_obj,
                    image_path,
                    quality=image_file.resolution[0],
                    media_type=None,
                    codec_or_mime=image_file.mime):
                print(f"Media download progress: {progress}%")
            assert os.path.exists(image_path)
            assert os.stat(image_path).st_size == image_file.size

def test_download_video(host, token, video):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)

    archival = video_obj.media_files.archival
    streaming = video_obj.media_files.streaming
    thumbnail = video_obj.media_files.thumbnail
    thumbnail_gif = video_obj.media_files.thumbnail_gif

    available = [x.resolution[0] for x in archival]
    best = max(available)
    best_idx = available.index(best)

    # Verify we get the best archival when we ask
    with tempfile.TemporaryDirectory() as td:
        video_path = os.path.join(td, video_obj.name)
        for progress in tator.download_media(
                tator_api,
                video_obj,
                video_path,
                quality=None,
                media_type=None):
            print(f"Media download progress: {progress}%")
        assert os.path.exists(video_path)
        assert os.stat(video_path).st_size == archival[best_idx].size

    for idx,video_file in enumerate(archival):
        with tempfile.TemporaryDirectory() as td:
            video_path = os.path.join(td, video_obj.name)
            for progress in tator.download_media(
                    tator_api,
                    video_obj,
                    video_path,
                    quality=video_file.resolution[0],
                    media_type='archival'):
                print(f"Media download progress: {progress}%")
            assert os.path.exists(video_path)
            assert os.stat(video_path).st_size == video_file.size



    # Verify we can download the best when we ask (streaming)
    available = [x.resolution[0] for x in streaming]
    best = max(available)
    best_idx = available.index(best)

    # Verify we get the best streaming when we ask
    with tempfile.TemporaryDirectory() as td:
        video_path = os.path.join(td, video_obj.name)
        for progress in tator.download_media(
                tator_api,
                video_obj,
                video_path,
                quality=None,
                media_type='streaming'):
            print(f"Media download progress: {progress}%")
        assert os.path.exists(video_path)
        assert os.stat(video_path).st_size == streaming[best_idx].size


    # Verify we can download each file size
    for video_file in streaming:
        with tempfile.TemporaryDirectory() as td:
            video_path = os.path.join(td, video_obj.name)
            for progress in tator.download_media(
                    tator_api,
                    video_obj,
                    video_path,
                    quality=video_file.resolution[0],
                    media_type='streaming'):
                print(f"Media download progress: {progress}%")
            assert os.path.exists(video_path)
            assert os.stat(video_path).st_size == video_file.size


    available = [x.resolution[0] for x in thumbnail]
    best = max(available)
    best_idx = available.index(best)
    with tempfile.TemporaryDirectory() as td:
        image_path = os.path.join(td, video_obj.name)
        for progress in tator.download_media(
                tator_api,
                video_obj,
                image_path,
                quality=None,
                media_type='thumbnail'):
            print(f"Media download progress: {progress}%")
        assert os.path.exists(image_path)
        assert os.stat(image_path).st_size == thumbnail[best_idx].size

    available = [x.resolution[0] for x in thumbnail_gif]
    best = max(available)
    best_idx = available.index(best)
    with tempfile.TemporaryDirectory() as td:
        image_path = os.path.join(td, video_obj.name)
        for progress in tator.download_media(
                tator_api,
                video_obj,
                image_path,
                quality=None,
                media_type='thumbnail_gif'):
            print(f"Media download progress: {progress}%")
        assert os.path.exists(image_path)
        assert os.stat(image_path).st_size == thumbnail_gif[best_idx].size



# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class BucketGCPConfig(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'auth_provider_x509_cert_url': 'str',
        'auth_uri': 'str',
        'client_email': 'str',
        'client_id': 'str',
        'client_x509_cert_url': 'str',
        'private_key': 'str',
        'private_key_id': 'str',
        'project_id': 'str',
        'token_uri': 'str',
        'type': 'str'
    }

    attribute_map = {
        'auth_provider_x509_cert_url': 'auth_provider_x509_cert_url',
        'auth_uri': 'auth_uri',
        'client_email': 'client_email',
        'client_id': 'client_id',
        'client_x509_cert_url': 'client_x509_cert_url',
        'private_key': 'private_key',
        'private_key_id': 'private_key_id',
        'project_id': 'project_id',
        'token_uri': 'token_uri',
        'type': 'type'
    }

    def __init__(self, auth_provider_x509_cert_url=None, auth_uri=None, client_email=None, client_id=None, client_x509_cert_url=None, private_key=None, private_key_id=None, project_id=None, token_uri=None, type=None, local_vars_configuration=None):  # noqa: E501
        """BucketGCPConfig - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._auth_provider_x509_cert_url = None
        self._auth_uri = None
        self._client_email = None
        self._client_id = None
        self._client_x509_cert_url = None
        self._private_key = None
        self._private_key_id = None
        self._project_id = None
        self._token_uri = None
        self._type = None
        self.discriminator = None

        self.auth_provider_x509_cert_url = auth_provider_x509_cert_url
        self.auth_uri = auth_uri
        self.client_email = client_email
        self.client_id = client_id
        self.client_x509_cert_url = client_x509_cert_url
        self.private_key = private_key
        self.private_key_id = private_key_id
        self.project_id = project_id
        self.token_uri = token_uri
        self.type = type

    @property
    def auth_provider_x509_cert_url(self):
        """
        Provider cert url.

        :return: The auth_provider_x509_cert_url of this BucketGCPConfig. 
        :rtype: str
        """
        return self._auth_provider_x509_cert_url

    @auth_provider_x509_cert_url.setter
    def auth_provider_x509_cert_url(self, auth_provider_x509_cert_url):
        """
        Provider cert url.

        :param auth_provider_x509_cert_url: The auth_provider_x509_cert_url of this BucketGCPConfig.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and auth_provider_x509_cert_url is None:  # noqa: E501
            raise ValueError("Invalid value for `auth_provider_x509_cert_url`, must not be `None`")  # noqa: E501

        self._auth_provider_x509_cert_url = auth_provider_x509_cert_url

    @property
    def auth_uri(self):
        """
        Authorization URI.

        :return: The auth_uri of this BucketGCPConfig. 
        :rtype: str
        """
        return self._auth_uri

    @auth_uri.setter
    def auth_uri(self, auth_uri):
        """
        Authorization URI.

        :param auth_uri: The auth_uri of this BucketGCPConfig.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and auth_uri is None:  # noqa: E501
            raise ValueError("Invalid value for `auth_uri`, must not be `None`")  # noqa: E501

        self._auth_uri = auth_uri

    @property
    def client_email(self):
        """
        Email address associated with the service account.

        :return: The client_email of this BucketGCPConfig. 
        :rtype: str
        """
        return self._client_email

    @client_email.setter
    def client_email(self, client_email):
        """
        Email address associated with the service account.

        :param client_email: The client_email of this BucketGCPConfig.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and client_email is None:  # noqa: E501
            raise ValueError("Invalid value for `client_email`, must not be `None`")  # noqa: E501

        self._client_email = client_email

    @property
    def client_id(self):
        """
        ID of the client account.

        :return: The client_id of this BucketGCPConfig. 
        :rtype: str
        """
        return self._client_id

    @client_id.setter
    def client_id(self, client_id):
        """
        ID of the client account.

        :param client_id: The client_id of this BucketGCPConfig.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and client_id is None:  # noqa: E501
            raise ValueError("Invalid value for `client_id`, must not be `None`")  # noqa: E501

        self._client_id = client_id

    @property
    def client_x509_cert_url(self):
        """
        Client cert url.

        :return: The client_x509_cert_url of this BucketGCPConfig. 
        :rtype: str
        """
        return self._client_x509_cert_url

    @client_x509_cert_url.setter
    def client_x509_cert_url(self, client_x509_cert_url):
        """
        Client cert url.

        :param client_x509_cert_url: The client_x509_cert_url of this BucketGCPConfig.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and client_x509_cert_url is None:  # noqa: E501
            raise ValueError("Invalid value for `client_x509_cert_url`, must not be `None`")  # noqa: E501

        self._client_x509_cert_url = client_x509_cert_url

    @property
    def private_key(self):
        """
        Account access key.

        :return: The private_key of this BucketGCPConfig. 
        :rtype: str
        """
        return self._private_key

    @private_key.setter
    def private_key(self, private_key):
        """
        Account access key.

        :param private_key: The private_key of this BucketGCPConfig.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and private_key is None:  # noqa: E501
            raise ValueError("Invalid value for `private_key`, must not be `None`")  # noqa: E501

        self._private_key = private_key

    @property
    def private_key_id(self):
        """
        Account secret key.

        :return: The private_key_id of this BucketGCPConfig. 
        :rtype: str
        """
        return self._private_key_id

    @private_key_id.setter
    def private_key_id(self, private_key_id):
        """
        Account secret key.

        :param private_key_id: The private_key_id of this BucketGCPConfig.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and private_key_id is None:  # noqa: E501
            raise ValueError("Invalid value for `private_key_id`, must not be `None`")  # noqa: E501

        self._private_key_id = private_key_id

    @property
    def project_id(self):
        """
        GCP project id.

        :return: The project_id of this BucketGCPConfig. 
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """
        GCP project id.

        :param project_id: The project_id of this BucketGCPConfig.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and project_id is None:  # noqa: E501
            raise ValueError("Invalid value for `project_id`, must not be `None`")  # noqa: E501

        self._project_id = project_id

    @property
    def token_uri(self):
        """
        Token URI.

        :return: The token_uri of this BucketGCPConfig. 
        :rtype: str
        """
        return self._token_uri

    @token_uri.setter
    def token_uri(self, token_uri):
        """
        Token URI.

        :param token_uri: The token_uri of this BucketGCPConfig.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and token_uri is None:  # noqa: E501
            raise ValueError("Invalid value for `token_uri`, must not be `None`")  # noqa: E501

        self._token_uri = token_uri

    @property
    def type(self):
        """
        Type of account, should be `service_account`.

        :return: The type of this BucketGCPConfig. 
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """
        Type of account, should be `service_account`.

        :param type: The type of this BucketGCPConfig.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self._type = type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BucketGCPConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, BucketGCPConfig):
            return True

        return self.to_dict() != other.to_dict()



# Test alternate bucket upload, only runs on compose
import pytest
import tator
from tator.util._upload_file import _upload_file
from tator.transcode.transcode import make_video_definition
import tempfile
import requests
import os


def test_alt_bucket_upload(alt_bucket, host, token, project, video_type):
    api = tator.get_api(host, token)

    media_specs = [
        {
            "type": video_type,
            "section": "alt_bucket",
            "name": f"count.mp4",
            "md5": "foo",
        }
    ]

    response = api.create_media_list(project, body=media_specs)
    media_id = response.id[0]

    VIDEO_URL = "https://github.com/cvisionai/rgb_test_videos/raw/v0.0.3/samples/count.mp4"

    file_size = 0
    with tempfile.TemporaryDirectory() as td:
        r = requests.get(VIDEO_URL)
        video_path = os.path.join(td, "count.mp4")
        f = open(video_path, "wb")
        for chunk in r.iter_content(chunk_size=1 * 1024 * 1024):
            if chunk:  # filter out keep-alive new chunks
                f.write(chunk)
        f.close()
        file_size = os.path.getsize(video_path)

        filename = "count.mp4"
        for _, upload_info in _upload_file(
            api,
            project,
            video_path,
            media_id=media_id,
            filename=filename,
            chunk_size=0x10000000,
            bucket_id=alt_bucket,
        ):
            pass
        media_def = {
            **make_video_definition(video_path),
            "path": upload_info.key,
            "bucket_id": alt_bucket,
        }
        # Patch in video file with the api.
        response = api.create_video_file(media_id, role="archival", video_definition=media_def)

    # Check that the video file was uploaded to alt_bucket
    media = api.get_media(media_id, presigned=3600)
    path = media.media_files.archival[0].path
    assert path.find("tator-alt") >= 0

    with tempfile.TemporaryDirectory() as td:
        download_path = os.path.join(td, "count.mp4")
        with open(download_path, "wb") as f:
            r = requests.get(path)
            for chunk in r.iter_content(chunk_size=1 * 1024 * 1024):
                f.write(chunk)

        assert os.path.getsize(download_path) == file_size



import os
import tator
import tempfile

from tator.extractor.extractor import process_file

def test_extract(host, token, project, video_temp, box_type,
                 image_type,
                 state_type):
    video = video_temp
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)
    localization={'x':0.50,
                  'y':0.50,
                  'width': 0.25,
                  'height': 0.25,
                  'type': box_type,
                  'media_id': video,
                  'frame': 0}
    state={'frame': 1, 'media_ids':[video], 'type': state_type}
    api = tator.get_api(host,token)
    response = api.create_localization_list(project, [localization])
    localization_id = response.id[0]
    response = api.create_state_list(project, [state])
    state_id = response.id[0]

    with tempfile.TemporaryDirectory() as td:
        out = os.path.join(td, "temp.mp4")
        for _ in tator.download_media(api, api.get_media(video), out):
            pass
        
        process_file(api,
                     project,
                     out,
                     api.get_media(video),
                     'localization_thumbnail',
                     api.get_localization_list(project,media_id=[video]),
                     td)

        process_file(api,
                     project,
                     out,
                     api.get_media(video),
                     'localization_keyframe',
                     api.get_localization_list(project,media_id=[video]),
                     td)

        process_file(api,
                     project,
                     out,
                     api.get_media(video),
                     'state',
                     api.get_state_list(project,media_id=[video]),
                     td)

        image_cnt = 0
        for _, __, files in os.walk(td):
            for fp in files:
                if os.path.splitext(fp)[1] == '.png':
                    image_cnt += 1


        assert image_cnt == 3

    # Clean up.
    api.delete_localization(localization_id)
    api.delete_state(state_id)




# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class StateSpec(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'attributes': 'dict(str, object)',
        'elemental_id': 'str',
        'frame': 'int',
        'localization_ids': 'list[int]',
        'mark': 'int',
        'media_ids': 'list[int]',
        'parent': 'float',
        'type': 'int',
        'user_elemental_id': 'str',
        'version': 'int'
    }

    attribute_map = {
        'attributes': 'attributes',
        'elemental_id': 'elemental_id',
        'frame': 'frame',
        'localization_ids': 'localization_ids',
        'mark': 'mark',
        'media_ids': 'media_ids',
        'parent': 'parent',
        'type': 'type',
        'user_elemental_id': 'user_elemental_id',
        'version': 'version'
    }

    def __init__(self, attributes=None, elemental_id=None, frame=None, localization_ids=None, mark=None, media_ids=None, parent=None, type=None, user_elemental_id=None, version=None, local_vars_configuration=None):  # noqa: E501
        """StateSpec - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._attributes = None
        self._elemental_id = None
        self._frame = None
        self._localization_ids = None
        self._mark = None
        self._media_ids = None
        self._parent = None
        self._type = None
        self._user_elemental_id = None
        self._version = None
        self.discriminator = None

        if attributes is not None:
            self.attributes = attributes
        self.elemental_id = elemental_id
        if frame is not None:
            self.frame = frame
        if localization_ids is not None:
            self.localization_ids = localization_ids
        if mark is not None:
            self.mark = mark
        self.media_ids = media_ids
        self.parent = parent
        self.type = type
        if user_elemental_id is not None:
            self.user_elemental_id = user_elemental_id
        if version is not None:
            self.version = version

    @property
    def attributes(self):
        """
        Object containing attribute values.

        :return: The attributes of this StateSpec. 
        :rtype: dict(str, object)
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """
        Object containing attribute values.

        :param attributes: The attributes of this StateSpec.
        :type: dict(str, object)
        """

        self._attributes = attributes

    @property
    def elemental_id(self):
        """
        The elemental ID of the object.

        :return: The elemental_id of this StateSpec. 
        :rtype: str
        """
        return self._elemental_id

    @elemental_id.setter
    def elemental_id(self, elemental_id):
        """
        The elemental ID of the object.

        :param elemental_id: The elemental_id of this StateSpec.
        :type: str
        """

        self._elemental_id = elemental_id

    @property
    def frame(self):
        """
        Frame number this state applies to.

        :return: The frame of this StateSpec. 
        :rtype: int
        """
        return self._frame

    @frame.setter
    def frame(self, frame):
        """
        Frame number this state applies to.

        :param frame: The frame of this StateSpec.
        :type: int
        """

        self._frame = frame

    @property
    def localization_ids(self):
        """
        List of localization IDs that this state applies to.

        :return: The localization_ids of this StateSpec. 
        :rtype: list[int]
        """
        return self._localization_ids

    @localization_ids.setter
    def localization_ids(self, localization_ids):
        """
        List of localization IDs that this state applies to.

        :param localization_ids: The localization_ids of this StateSpec.
        :type: list[int]
        """

        self._localization_ids = localization_ids

    @property
    def mark(self):
        """
        Revision number of this object on this version branch

        :return: The mark of this StateSpec. 
        :rtype: int
        """
        return self._mark

    @mark.setter
    def mark(self, mark):
        """
        Revision number of this object on this version branch

        :param mark: The mark of this StateSpec.
        :type: int
        """

        self._mark = mark

    @property
    def media_ids(self):
        """
        List of media IDs that this state applies to.

        :return: The media_ids of this StateSpec. 
        :rtype: list[int]
        """
        return self._media_ids

    @media_ids.setter
    def media_ids(self, media_ids):
        """
        List of media IDs that this state applies to.

        :param media_ids: The media_ids of this StateSpec.
        :type: list[int]
        """
        if self.local_vars_configuration.client_side_validation and media_ids is None:  # noqa: E501
            raise ValueError("Invalid value for `media_ids`, must not be `None`")  # noqa: E501

        self._media_ids = media_ids

    @property
    def parent(self):
        """
        If a clone, the pk of the parent.

        :return: The parent of this StateSpec. 
        :rtype: float
        """
        return self._parent

    @parent.setter
    def parent(self, parent):
        """
        If a clone, the pk of the parent.

        :param parent: The parent of this StateSpec.
        :type: float
        """

        self._parent = parent

    @property
    def type(self):
        """
        Unique integer identifying a state type.

        :return: The type of this StateSpec. 
        :rtype: int
        """
        return self._type

    @type.setter
    def type(self, type):
        """
        Unique integer identifying a state type.

        :param type: The type of this StateSpec.
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self._type = type

    @property
    def user_elemental_id(self):
        """
        Unique ID of the original user who created this. If permissions allow, will change the creating user to the one referenced by this elemental_id

        :return: The user_elemental_id of this StateSpec. 
        :rtype: str
        """
        return self._user_elemental_id

    @user_elemental_id.setter
    def user_elemental_id(self, user_elemental_id):
        """
        Unique ID of the original user who created this. If permissions allow, will change the creating user to the one referenced by this elemental_id

        :param user_elemental_id: The user_elemental_id of this StateSpec.
        :type: str
        """

        self._user_elemental_id = user_elemental_id

    @property
    def version(self):
        """
        Unique integer identifying the version.

        :return: The version of this StateSpec. 
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """
        Unique integer identifying the version.

        :param version: The version of this StateSpec.
        :type: int
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StateSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, StateSpec):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class FloatArrayQuery(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'center': 'list[float]',
        'metric': 'str',
        'name': 'str',
        'order': 'str'
    }

    attribute_map = {
        'center': 'center',
        'metric': 'metric',
        'name': 'name',
        'order': 'order'
    }

    def __init__(self, center=None, metric='l2norm', name=None, order='asc', local_vars_configuration=None):  # noqa: E501
        """FloatArrayQuery - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._center = None
        self._metric = None
        self._name = None
        self._order = None
        self.discriminator = None

        self.center = center
        if metric is not None:
            self.metric = metric
        self.name = name
        if order is not None:
            self.order = order

    @property
    def center(self):
        """
        Center of the query.

        :return: The center of this FloatArrayQuery. 
        :rtype: list[float]
        """
        return self._center

    @center.setter
    def center(self, center):
        """
        Center of the query.

        :param center: The center of this FloatArrayQuery.
        :type: list[float]
        """
        if self.local_vars_configuration.client_side_validation and center is None:  # noqa: E501
            raise ValueError("Invalid value for `center`, must not be `None`")  # noqa: E501

        self._center = center

    @property
    def metric(self):
        """
        Distance metric from center of query.

        :return: The metric of this FloatArrayQuery. 
        :rtype: str
        """
        return self._metric

    @metric.setter
    def metric(self, metric):
        """
        Distance metric from center of query.

        :param metric: The metric of this FloatArrayQuery.
        :type: str
        """
        allowed_values = ["l2norm", "ip", "cosine"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and metric not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `metric` ({0}), must be one of {1}"  # noqa: E501
                .format(metric, allowed_values)
            )

        self._metric = metric

    @property
    def name(self):
        """
        Name of the attribute.

        :return: The name of this FloatArrayQuery. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the attribute.

        :param name: The name of this FloatArrayQuery.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def order(self):
        """
        Order in which results should be returned.

        :return: The order of this FloatArrayQuery. 
        :rtype: str
        """
        return self._order

    @order.setter
    def order(self, order):
        """
        Order in which results should be returned.

        :param order: The order of this FloatArrayQuery.
        :type: str
        """
        allowed_values = ["asc", "desc"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and order not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `order` ({0}), must be one of {1}"  # noqa: E501
                .format(order, allowed_values)
            )

        self._order = order

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, FloatArrayQuery):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, FloatArrayQuery):
            return True

        return self.to_dict() != other.to_dict()
